# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import Attrs, Clock, Connector, Pins, Resource
from torii.platform.resources     import ButtonResources, LEDResources, SPIFlashResources
from torii.platform.vendor.xilinx import XilinxPlatform

__all__ = (
	'NumatoMimasPlatform',
)

class NumatoMimasPlatform(XilinxPlatform):
	device      = 'xc6slx9'
	package     = 'tqg144'
	speed       = '2'
	default_clk = 'clk100'

	pretty_name = 'Mimas S6'
	description = 'Numato Mimas S6 Xilinx Spartan6 Development Board'

	resources   = [
		Resource('clk100', 0, Pins('P126', dir = 'i'), Clock(100e6), Attrs(IOSTANDARD = 'LVCMOS33')),

		*LEDResources(
			pins = 'P119 P118 P117 P116 P115 P114 P112 P111',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*ButtonResources(
			pins = 'P124 P123 P121 P120',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33', PULLUP = 'TRUE')
		),

		*SPIFlashResources(0,
			cs_n = 'P38', clk = 'P70', copi = 'P64', cipo = '65',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
	]
	connectors  = [
		Connector('p', 1,
			'-    -    P35  P34  P33  P32  P30  P29  P27  P26  '
			'P24  P23  P22  P21  P17  P16  P15  P14  P12  P11  '
			'P10  P9   P8   P7   P6   P5   P2   P1   P142 P141 '
			'P140 P139 P138 P137 P134 P133 P132 P131 -    -    '
		),
		Connector('p', 2,
			'-    -    P43  P44  P45  P46  P47  P48  P50  P51  '
			'P55  P56  P74  P75  P78  P79  P80  P81  -    -    '
			'P82  P83  P84  P85  P87  P88  P92  P93  P94  P95  '
			'P97  P98  P99  P100 P101 P102 P104 P105 -    -    '
		)
	]

	# Programming this board is not currently supported.


if __name__ == '__main__':
	from ..test.blinky import Blinky
	NumatoMimasPlatform().build(Blinky())



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import Attrs, Clock, Connector, Pins, Resource
from torii.platform.resources     import LEDResources, SPIFlashResources
from torii.platform.vendor.xilinx import XilinxPlatform

__all__ = (
	'TE0714_03_50_2IPlatform',
)

class TE0714_03_50_2IPlatform(XilinxPlatform):
	device      = 'xc7a50t'
	package     = 'csg325'
	speed       = '2'
	default_clk = 'clk25'

	pretty_name = 'TE0714-03-50'
	description = 'Trenz Electronic TE0714-03-50 Xilinx Artix7-50T based FPGA Module'

	resources   = [
		Resource('clk25', 0, Pins('T14', dir = 'i'), Clock(25e6), Attrs(IOSTANDARD = 'LVCMOS18')),

		*LEDResources(pins = 'K18', attrs = Attrs(IOSTANDARD = 'LVCMOS18')),

		*SPIFlashResources(0,
			cs_n = 'L15', clk = 'E8', copi = 'K16', cipo = 'K17', wp_n = 'J15', hold_n = 'J16',
			attrs = Attrs(IOSTANDARD = 'LVCMOS18')
		)
	]
	connectors = [
		Connector('JM1', 0,
			'G4  D6  '
			'G3  D5  '
			'-   -   '
			'C4  B2  '
			'C3  B1  '
			'-   -   '
			'A4  D2  '
			'A3  D1  '
			'-   -   '
			'E4  F2  '
			'E3  F1  '
			'-   -   '
			'K10 H2  '
			'L9  H1  '
			'-   -   '
			'J5  L5  '
			'J4  M5  '
			'K6  M2  '
			'K5  M1  '
			'K3  K2  '
			'L2  K1  '
			'L4  N1  '
			'L3  P1  '
			'-   -   '
			'M4  M6  '
			'N4  N6  '
			'N2  R2  '
			'N3  R1  '
			'P3  R3  '
			'P4  T2  '
			'L6  U1  '
			'T3  U2  '
			'T4  -   '
			'R5  V2  '
			'T5  V3  '
			'P5  U4  '
			'P6  V4  '
			'T7  U5  '
			'R7  U6  '
			'V6  V7  '
			'U7  V8  '
			'-   -   '
			'V9  T9  '
			'U9  T8  '
			'V11 F8  '
			'U11 R8  '
			'V12 -   '
			'V13 F12 '
			'-   -   '
			'-   -   '
		),
		Connector('JM2', 0,
			'A12 B9  '
			'B12 A9  '
			'A13 B10 '
			'A14 A10 '
			'B14 B11 '
			'A15 C11 '
			'C14 C8  '
			'B15 D8  '
			'-   E11 '
			'A17 C9  '
			'B16 D9  '
			'B17 D11 '
			'C16 C12 '
			'C18 D13 '
			'C17 C13 '
			'D18 E13 '
			'E17 D14 '
			'-   -   '
			'E18 D15 '
			'F17 E15 '
			'F18 D16 '
			'G17 E16 '
			'F15 F14 '
			'G15 G14 '
			'H17 G16 '
			'H18 H16 '
			'-   A16 '
			'K17 J14 '
			'L18 K15 '
			'M17 M14 '
			'M16 N14 '
			'R18 N16 '
			'T18 N17 '
			'E8  K16 '
			'L15 J16 '
			'L17 -   '
			'J15 P16 '
			'N18 P15 '
			'P18 R17 '
			'T17 R16 '
			'U17 P14 '
			'-   R15 '
			'V17 T15 '
			'V16 T14 '
			'U16 -   '
			'U15 R13 '
			'U14 T13 '
			'V14 U12 '
			'U10 T12 '
			'L14 R12 '
		)
	]


if __name__ == '__main__':
	from ..test.blinky import Blinky
	TE0714_03_50_2IPlatform().build(Blinky(), do_program = False)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Attrs, Clock, Connector, Pins, PinsN, Resource
from torii.build.run                     import BuildProducts
from torii.platform.resources            import LEDResources, SPIFlashResources, SwitchResources
from torii.platform.vendor.lattice.ice40 import ICE40Platform

__all__ = (
	'ICE40UP5KBEVNPlatform',
)

class ICE40UP5KBEVNPlatform(ICE40Platform):
	device      = 'iCE40UP5K'
	package     = 'SG48'
	default_clk = 'clk12'

	pretty_name = 'ICE40UP5KEVN'
	description = 'Lattice iCE40-UP5K Evaluation Board'

	resources   = [
		# J51 must be connected to use clk12 (it is by default)
		Resource(
			'clk12', 0, Pins('35', dir = 'i'), Clock(12e6),
			Attrs(GLOBAL = True, IO_STANDARD = 'SB_LVCMOS')
		),

		# 3 LEDs are present in an RGB common-anode package.
		*LEDResources(
			pins = '39 40 41', invert = True,
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		Resource('led_b', 0, PinsN('39', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_g', 0, PinsN('40', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_r', 0, PinsN('41', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),

		# 4 DIP switches are available, requiring internal pull-ups.
		# The switches' 'ON' label points to the position which
		# connects them to ground, so invert the inputs.
		*SwitchResources(
			pins = '23 25 34 43', invert = True,
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS', PULLUP = 1)
		),

		*SPIFlashResources(0,
			cs_n = '16', clk = '15', copi = '14', cipo = '17',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
	]
	connectors  = [
		Connector('aardvark', 0, # J1
			'- - - - 14 - 15 17 16 -'
		),
		Connector('pmod', 0, # U6 (board), U11 (schematic)
			'16 14 17 15 - - 27 26 32 31 - -'
		),
		Connector('j', 0, # 'Header A' (J52)
			'- - 39 14 40 17 - 15 41 16 - -'
		),
		Connector('j', 1, # 'Header B' (J2)
			'- - 23 - 25 - 26 36 27 42 32 38 31 28 37 15 34 - 43 -'
		),
		Connector('j', 2, # 'Header C' (J3)
			'- 12 3 21 3 13 48 20 45 19 47 18 44 11 46 10 2 9 - 6'
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		iceprog = environ.get('ICEPROG', 'iceprog')
		with products.extract(f'{name}.bin') as bitstream_fn:
			check_call([iceprog, bitstream_fn])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	ICE40UP5KBEVNPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import Attrs, Connector, Pins, Resource, Subsignal
from torii.build.run              import BuildProducts
from torii.platform.resources     import I2CResource, LEDResources, SDCardResources, UARTResource
from torii.platform.vendor.altera import AlteraPlatform

__all__ = (
	'Chameleon96Platform',
)

class Chameleon96Platform(AlteraPlatform):
	device      = '5CSEBA6' # Cyclone V SE 110K LEs
	package     = 'U19'     # UBGA-484
	speed       = 'I7'
	default_clk = 'cyclonev_oscillator'

	pretty_name = 'Chameleon96'
	description = 'Altera Cyclone V SoC Development Board'

	resources   = [
		# WIFI and BT LEDs
		*LEDResources(
			pins = 'Y19 Y20',
			attrs = Attrs(io_standard = '2.5 V')
		),

		# TDA19988 HDMI transmitter
		Resource('tda19988', 0,
			Subsignal('vpa',       Pins('    W8  W7  V6  V5  U6 ', dir = 'o')), # bits 3 to 7
			Subsignal('vpb',       Pins('AB5 AA5 AA8 AB8 AB9 Y11', dir = 'o')), # bits 2 to 7
			Subsignal('vpc',       Pins('    W6  Y5  AB7 AA7 AA6', dir = 'o')), # bits 3 to 7
			Subsignal('pclk',      Pins('AB10', dir = 'o')),
			Subsignal('hsync',     Pins('V10 ', dir = 'o')),
			Subsignal('vsync',     Pins('V7  ', dir = 'o')),
			Subsignal('de',        Pins('Y8  ', dir = 'o')),
			Attrs(io_standard = '1.8 V')
		),

		I2CResource('tda19988_i2c', 0,
			scl = 'U7', sda = 'U10',
			attrs = Attrs(io_standard = '1.8 V'),
		),

		Resource('tda19988_i2s', 0,
			Subsignal('mclk',  Pins('V11 ', dir = 'o')), # OSC_IN/AP3
			Subsignal('txd',   Pins('AA11', dir = 'o')), # AP1
			Subsignal('txc',   Pins('W11 ', dir = 'o')), # ACLK
			Subsignal('txfs',  Pins('V9  ', dir = 'o')), # AP0
			Attrs(io_standard = '1.8 V')
		),

		# Wifi and BT module
		*SDCardResources('wifi', 0,
			clk = 'AB20', cmd = 'AB18', dat0 = 'Y14', dat1 = 'AB19', dat2 = 'AA18', dat3 = 'AB15',
			attrs = Attrs(io_standard = '1.8 V'),
		),

		Resource('bt', 0,
			Subsignal('host_wake', Pins('AB12', dir = 'o')),
			Attrs(io_standard = '1.8 V'),
		),

		Resource('bt_i2s', 0,
			Subsignal('txd',   Pins('Y15 ', dir = 'o')), # BT_PCM_IN
			Subsignal('rxd',   Pins('Y16 ', dir = 'i')), # BT_PCM_OUT
			Subsignal('txc',   Pins('AA13', dir = 'i')), # BT_PCM_CLK
			Subsignal('txfs',  Pins('AB13', dir = 'i')), # BT_PCM_SYNC
			Attrs(io_standard = '1.8 V'),
		),

		UARTResource('bt_uart', 0,
			rx = 'AB14', cts = 'AB17', tx = 'AA15', rts = 'AA16', role = 'dte',
			attrs = Attrs(io_standard = '1.8 V'),
		),
	]

	connectors  = [
		# J3, 2x20 expansion port
		Connector('J', 3,
			'-      -      Y13    -      W14    -      C5     -      C6     -'
			'-      -      -      -      -      E5     -      F5     -      A6'
			'-      A5     -      -      -      -      -      -      -      -'
			'-      -      -      -      -      -      -      -      -      -'
		),

		# J8, 2x30 high speed expansion port (MIPI CSI)
		Connector('J', 8,
			'-      V16    -      U17    -      -      -      V17    -      W18'
			'-      -      -      U18    W12    V19    -      -      -      -'
			'-      -      -      -      -      -      -      -      -      -'
			'-      -      -      -      -      -      -      -      -      -'
			'-      -      -      -      -      -      -      -      -      -'
			'-      -      -      -      -      -      -      -      -      -'
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			# The @2 selects the second device in the JTAG chain, because this chip
			# puts the ARM cores first.
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename + '@2'
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	Chameleon96Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

import os
import subprocess

from torii.build                         import Attrs, Clock, Connector, Pins, Resource
from torii.platform.resources            import RGBLEDResource, SPIFlashResources
from torii.platform.vendor.lattice.ice40 import ICE40Platform

__all__ = (
	'UpduinoV3Platform',
)

class UpduinoV3Platform(ICE40Platform):
	device      = 'iCE40UP5K'
	package     = 'SG48'
	default_clk = 'SB_HFOSC'
	hfosc_div   = 0

	pretty_name = 'Upduino V3'
	description = 'TinyVision Upduino V3 Lattice iCE40-UP5K Development Board'

	resources   = [
		# Solder the OSC jumper to connect the onboard oscillator to pin 20.
		# Note that this overlaps with the QSPI pins.
		Resource(
			'clk12', 0, Pins('20', dir = 'i'), Clock(12e6), Attrs(IO_STANDARD = 'SB_LVCMOS')
		),

		RGBLEDResource(0,
			r = '41', g = '39', b = '40', invert = True,
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),

		# To use QSPI mode, solder the appropriate jumpers.
		*SPIFlashResources(0,
			cs_n = '16', clk = '15', cipo = '17', copi = '14', wp_n = '10', hold_n = '20',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
	]
	connectors  = [
		# 'Left' row of header pins (JP5 on the schematic)
		Connector('j', 0, '41 39 40 - - - 23 25 26 27 32 35 31 37 34 43 36 42 38 28'),
		# 'Right' row of header pins (JP6 on the schematic)
		Connector('j', 1, '20 10 - - 12 21 13 19 18 11 9 6 44 4 3 48 45 47 46 2')
	]

	def toolchain_program(self, products, name):
		iceprog = os.environ.get('ICEPROG', 'iceprog')
		with products.extract(f'{name}.bin') as bitstream_filename:
			subprocess.check_call([iceprog, bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	UpduinoV3Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause
import datetime
import os
import sys
from pathlib import Path

sys.path.insert(0, os.path.abspath('.'))

from torii_boards import __version__ as boards_version

ROOT_DIR = (Path(__file__).parent).parent


project = 'Torii-HDL Boards'
version = boards_version
release = version.split('+')[0]
copyright = f'{datetime.date.today().year}, Shrine Maiden Heavy Industries'
language  = 'en'
docver    = version

extensions = [
	'sphinx.ext.autodoc',
	'sphinx.ext.extlinks',
	'sphinx.ext.githubpages',
	'sphinx.ext.intersphinx',
	'sphinx.ext.napoleon',
	'sphinx.ext.todo',
	'sphinx_inline_tabs',
	'myst_parser',
	'sphinx_copybutton',
	'sphinx_design',
]


source_suffix = {
	'.rst': 'restructuredtext',
	'.md': 'markdown',
}

extlinks = {
	'issue': ('https://github.com/shrine-maiden-heavy-industries/torii-boards/issues/%s', 'torii-boards/%s'),
	'pypi':  ('https://pypi.org/project/%s/', '%s'),
}

pygments_style         = 'default'
pygments_dark_style    = 'monokai'
autodoc_member_order   = 'bysource'
todo_include_todos     = True

intersphinx_mapping = {
	'python': ('https://docs.python.org/3', None),
	'torii': ('https://torii.shmdn.link/', None)
}


napoleon_google_docstring              = False
napoleon_numpy_docstring               = True
napoleon_use_ivar                      = True
napoleon_use_admonition_for_notes      = True
napoleon_use_admonition_for_examples   = True
napoleon_use_admonition_for_references = True
napoleon_custom_sections  = [
	('Attributes', 'params_style'),
	'Platform overrides'
]

myst_heading_anchors = 3

templates_path = [
	'_templates',
]

html_baseurl     = 'https://torii-boards.shmdn.link/'
html_theme       = 'furo'
html_copy_source = False

html_theme_options = {

}

html_static_path = [
	'_static'
]

html_css_files = [
	'css/styles.css'
]

linkcheck_retries = 2
linkcheck_workers = 1 # At the cost of speed try to prevent rate-limiting
linkcheck_ignore  = [
	'https://www.intel.com/content/www/us/en/products/details/fpga/development-tools/quartus-prime.html',
	'https://www.xilinx.com/products/design-tools/ise-design-suite.html',
	'https://www.xilinx.com/products/design-tools/vivado.html',
	# Digilent is being a big meanie
	'https://digilent.com/reference/programmable-logic/genesys-2/start',
	'https://digilent.com/reference/programmable-logic/atlys/start',
]



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildProducts
from torii.platform.resources           import RGBLEDResource, SPIFlashResources, UARTResource, ULPIResource
from torii.platform.vendor.lattice.ecp5 import ECP5Platform

__all__ = (
	'ECPIX585Platform',
	'ECPIX545Platform',
)

class _ECPIX5Platform(ECP5Platform):
	package     = 'BG554'
	speed       = '8'
	default_clk = 'clk100'
	default_rst = 'rst'

	resources   = [
		Resource('rst', 0, PinsN('AB1', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33')),
		Resource(
			'clk100', 0, Pins('K23', dir = 'i'), Clock(100e6), Attrs(IO_TYPE = 'LVCMOS33')
		),

		RGBLEDResource(0,
			r = 'T23', g = 'R21', b = 'T22', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),
		RGBLEDResource(1,
			r = 'U21', g = 'W21', b = 'T24', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),
		RGBLEDResource(2,
			r = 'K21', g = 'K24', b = 'M21', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),
		RGBLEDResource(3,
			r = 'P21', g = 'R23', b = 'P22', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),

		UARTResource(0,
			rx = 'R26', tx = 'R24',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),

		*SPIFlashResources(0,
			cs_n = 'AA2', clk = 'AE3', cipo = 'AE2', copi = 'AD2', wp_n = 'AF2', hold_n = 'AE1',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),

		Resource('eth_rgmii', 0,
			Subsignal('rst',     PinsN('C13', dir = 'o')),
			Subsignal('mdio',    Pins('A13', dir = 'io')),
			Subsignal('mdc',     Pins('C11', dir = 'o')),
			Subsignal('tx_clk',  Pins('A12', dir = 'o')),
			Subsignal('tx_ctrl', Pins('C9', dir = 'o')),
			Subsignal('tx_data', Pins('D8 C8 B8 A8', dir = 'o')),
			Subsignal('rx_clk',  Pins('E11', dir = 'i')),
			Subsignal('rx_ctrl', Pins('A11', dir = 'i')),
			Subsignal('rx_data', Pins('B11 A10 B10 A9', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),
		Resource('eth_int', 0, PinsN('B13', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33')),

		Resource('ddr3', 0,
			Subsignal('clk',    DiffPairs('H3', 'J3', dir = 'o'), Attrs(IO_TYPE = 'SSTL15D_I')),
			Subsignal('clk_en', Pins('P1', dir = 'o')),
			Subsignal('we',     PinsN('R3', dir = 'o')),
			Subsignal('ras',    PinsN('T3', dir = 'o')),
			Subsignal('cas',    PinsN('P2', dir = 'o')),
			Subsignal('a',      Pins('T5 M3 L3 V6 K2 W6 K3 L1 H2 L2 N1 J1 M1 K1 H1', dir = 'o')),
			Subsignal('ba',     Pins('U6 N3 N4', dir = 'o')),
			Subsignal('dqs',    DiffPairs('V4 V1', 'U5 U2', dir = 'io'),
								Attrs(IO_TYPE = 'SSTL15D_I', TERMINATION = 'OFF', DIFFRESISTOR = '100')),
			Subsignal('dq',     Pins('T4 W4 R4 W5 R6 P6 P5 P4 R1 W3 T2 V3 U3 W1 T1 W2', dir = 'io'),
								Attrs(TERMINATION = '75')),
			Subsignal('dm',     Pins('U4 U1', dir = 'o')),
			Subsignal('odt',    Pins('P3', dir = 'o')),
			Attrs(IO_TYPE = 'SSTL15_I', SLEWRATE = 'FAST')
		),

		Resource('sata', 0,
			Subsignal('tx', DiffPairs('AD16', 'AD17', dir = 'o')),
			Subsignal('rx', DiffPairs('AF15', 'AF16', dir = 'i')),
			Attrs(IO_TYPE = 'LVDS')
		),

		ULPIResource('ulpi', 0,
			data = 'M26 L25 L26 K25 K26 J23 P25 H25',
			clk = 'H24', clk_dir = 'i', dir = 'F22', nxt = 'F23',
			stp = 'H23', rst = 'E23', rst_invert = False,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', SLEWRATE = 'SLOW')
		),

		Resource('usbc_cfg', 0,
			Subsignal('scl', Pins('D24', dir = 'io')),
			Subsignal('sda', Pins('C24', dir = 'io')),
			Subsignal('dir', Pins('B23', dir = 'i')),
			Subsignal('id',  Pins('D23', dir = 'i')),
			Subsignal('int', PinsN('B24', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),

		Resource('usbc_mux', 0,
			Subsignal('en',    Pins('C23', dir = 'oe')),
			Subsignal('amsel', Pins('B26', dir = 'oe')),
			Subsignal('pol',   Pins('D26', dir = 'o')),
			Subsignal('lna',   DiffPairs( 'AF9', 'AF10', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS18D')),
			Subsignal('lnb',   DiffPairs('AD10', 'AD11', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS18D')),
			Subsignal('lnc',   DiffPairs( 'AD7',  'AD8', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS18D')),
			Subsignal('lnd',   DiffPairs( 'AF6',  'AF7', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS18D')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),
	]

	connectors  = [
		Connector('pmod', 0, 'T25 U25 U24 V24 - - T26 U26 V26 W26 - -'),
		Connector('pmod', 1, 'U23 V23 U22 V21 - - W25 W24 W23 W22 - -'),
		Connector('pmod', 2, 'J24 H22 E21 D18 - - K22 J21 H21 D22 - -'),
		Connector('pmod', 3, ' E4  F4  E6  H4 - -  F3  D4  D5  F5 - -'),
		Connector('pmod', 4, 'E26 D25 F26 F25 - - C26 C25 A25 A24 - -'),
		Connector('pmod', 5, 'D19 C21 B21 C22 - - D21 A21 A22 A23 - -'),
		Connector('pmod', 6, 'C16 B17 C18 B19 - - A17 A18 A19 C19 - -'),
		Connector('pmod', 7, 'D14 B14 E14 B16 - - C14 A14 A15 A16 - -'),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		tool = environ.get('OPENFPGALOADER', 'openFPGALoader')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([
				tool, '-c', 'ft2232', '-m', bitstream_filename
			])

class ECPIX545Platform(_ECPIX5Platform):
	device      = 'LFE5UM5G-45F'

	pretty_name = 'ECPIX5 45F'
	description = 'Lambda Concept ECPIX5 45F Lattice ECP5-5G-45F Development Board'

	resources   = [
		*_ECPIX5Platform.resources,

		# The IT6613E HDMI transmitter has access to 8 bits per color channel.
		Resource('it6613e', 0,
			Subsignal('rst',   PinsN('N6', dir = 'o')),
			Subsignal('scl',   Pins('C17', dir = 'io')),
			Subsignal('sda',   Pins('E17', dir = 'io')),
			Subsignal('pclk',  Pins('C1', dir = 'o')),
			Subsignal('vsync', Pins('A4', dir = 'o')),
			Subsignal('hsync', Pins('B4', dir = 'o')),
			Subsignal('de',    Pins('A3', dir = 'o')),
			Subsignal('d',
				Subsignal('b', Pins(' B3  C3  D3  B1  C2  D2 D1 E3', dir = 'o')),
				Subsignal('g', Pins(' E1  F2  F1 D17 D16 E16 J6 H6', dir = 'o')),
				Subsignal('r', Pins('E10 D11 D10 C10  D9  E8 H5 J4', dir = 'o')),
			),
			Subsignal('mclk',  Pins('E19', dir = 'o')),
			Subsignal('sck',   Pins('D6', dir = 'o')),
			Subsignal('ws',    Pins('C6', dir = 'o')),
			Subsignal('i2s',   Pins('A6 B6 A5 C5', dir = 'o')),
			Subsignal('int',   PinsN('C4', dir = 'i')),
			Attrs(IO_TYPE = 'LVTTL33')
		),
	]


class ECPIX585Platform(_ECPIX5Platform):
	device      = 'LFE5UM5G-85F'

	pretty_name = 'ECPIX5 85F'
	description = 'Lambda Concept ECPIX5 85F Lattice ECP5-5G-85F Development Board'

	resources   = [
		*_ECPIX5Platform.resources,

		# The IT6613E HDMI transmitter has access to 12 bits per color channel. The LFE5UM5G-85F
		# has an additional I/O bank which is used to provide the lower 4 bits of each channel.
		Resource('it6613e', 0,
			Subsignal('rst',   PinsN('N6', dir = 'o')),
			Subsignal('scl',   Pins('C17', dir = 'io')),
			Subsignal('sda',   Pins('E17', dir = 'io')),
			Subsignal('pclk',  Pins('C1', dir = 'o')),
			Subsignal('vsync', Pins('A4', dir = 'o')),
			Subsignal('hsync', Pins('B4', dir = 'o')),
			Subsignal('de',    Pins('A3', dir = 'o')),
			Subsignal('d',
				Subsignal('b', Pins('AD25 AC26 AB24 AB25  B3  C3  D3  B1  C2  D2 D1 E3', dir = 'o')),
				Subsignal('g', Pins('AA23 AA22 AA24 AA25  E1  F2  F1 D17 D16 E16 J6 H6', dir = 'o')),
				Subsignal('r', Pins('AD26 AE25 AF25 AE26 E10 D11 D10 C10  D9  E8 H5 J4', dir = 'o')),
			),
			Subsignal('mclk',  Pins('E19', dir = 'o')),
			Subsignal('sck',   Pins('D6', dir = 'o')),
			Subsignal('ws',    Pins('C6', dir = 'o')),
			Subsignal('i2s',   Pins('A6 B6 A5 C5', dir = 'o')),
			Subsignal('int',   PinsN('C4', dir = 'i')),
			Attrs(IO_TYPE = 'LVTTL33')
		),
	]


if __name__ == '__main__':
	import argparse

	from ..test.blinky import Blinky

	parser = argparse.ArgumentParser()
	parser.add_argument(
		'--variant',
		choices = ('85', '45'),
		default = '85',
		help = 'platform variant (default: %(default)s)'
	)

	args = parser.parse_args()
	if args.variant == '85':
		platform = ECPIX585Platform()
	else:
		platform = ECPIX545Platform()

	platform.build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildPlan, BuildProducts
from torii.hdl.ir                       import Fragment
from torii.platform.resources           import ButtonResources, LEDResources, SDRAMResource, UARTResource
from torii.platform.vendor.lattice.ecp5 import ECP5Platform

__all__ = (
	'Colorlight_5A75B_R70Platform',
)

class Colorlight_5A75B_R70Platform(ECP5Platform):
	device      = 'LFE5U-25F'
	package     = 'BG256'
	speed       = '6'
	default_clk = 'clk25'

	pretty_name = 'Colorlight 5A-75B'
	description = 'Colorlight 5A-75B Lattice ECP5-25F FPGA Board'

	resources   = [
		Resource('clk25', 0, Pins('P6', dir = 'i'), Clock(25e6), Attrs(IO_TYPE = 'LVCMOS33')),

		*LEDResources(
			pins = 'P11', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', DRIVE = '4')
		),

		*ButtonResources(
			pins = 'M13', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),

		UARTResource(0,
			tx = 'P11', # led (J19 DATA_LED-)
			rx = 'M13', # btn (J19 KEY+)
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),

		# SPIFlash (W25Q32JV)   1x/2x/4x speed
		Resource('spi_flash', 0,
			Subsignal('cs',   PinsN('N8', dir = 'o')),
			# Subsignal('clk', Pins(', dir = 'i')),             # driven through USRMCLK
			Subsignal('cipo', Pins('T8', dir = 'i')),           # Chip: DI/IO0
			Subsignal('copi', Pins('T7', dir = 'o')),           #       DO/IO1
			# Subsignal('wp', PinsN('unknown', dir = 'o')),     #          IO2 Todo
			# Subsignal('hold', PinsN('unknown', dir = 'o')),   #          IO3 Todo
			Attrs(IO_TYPE = 'LVCMOS33')
		),

		# 2x ESMT M12L16161A-5T 1M x 16bit 200MHz SDRAMs (organized as 1M x 32bit)
		# 2x WinBond W9816G6JH-6 1M x 16bit 166MHz SDRAMs (organized as 1M x 32bit) are lso reported
		SDRAMResource(0,
			clk = 'C6', we_n = 'C7', cas_n = 'E7', ras_n = 'D7',
			ba = 'A7', a = 'A9 E10 B12 D13 C12 D11 D10 E9 D9 B7 C8',
			dq = 'B13 C11 C10 A11 C9 E8  B6  B9  A6  B5  A5  B4  B3 C3  A2  B2 '
				'E2  D3  A4  E4  D4 C4  E5  D5  E6  D6  D8  A8  B8 B10 B11 E11',
			attrs = Attrs(PULLMODE = 'NONE', DRIVE = '4', SLEWRATE = 'FAST', IO_TYPE = 'LVCMOS33')
		),

		# Broadcom B50612D Gigabit Ethernet Transceiver
		Resource('eth_rgmii', 0,
			Subsignal('rst',     PinsN('P5', dir = 'o')),
			Subsignal('mdc',     Pins('P3', dir = 'o')),
			Subsignal('mdio',    Pins('T2', dir = 'io')),
			Subsignal('tx_clk',  Pins('M2', dir = 'o')),
			Subsignal('tx_ctl',  Pins('M3', dir = 'o')),
			Subsignal('tx_data', Pins('L1 L3 P2 L4', dir = 'o')),
			Subsignal('rx_clk',  Pins('M1', dir = 'i')),
			Subsignal('rx_ctl',  Pins('N6', dir = 'i')),
			Subsignal('rx_data', Pins('N1 M5 N5 M6', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),

		# Broadcom B50612D Gigabit Ethernet Transceiver
		Resource('eth_rgmii', 1,
			Subsignal('rst',     PinsN('P5', dir = 'o')),
			Subsignal('mdc',     Pins('P3', dir = 'o')),
			Subsignal('mdio',    Pins('T2', dir = 'io')),
			Subsignal('tx_clk',  Pins('M12', dir = 'o')),
			Subsignal('tx_ctl',  Pins('R15', dir = 'o')),
			Subsignal('tx_data', Pins('T14 R12 R13 R14', dir = 'o')),
			Subsignal('rx_clk',  Pins('M16', dir = 'i')),
			Subsignal('rx_ctl',  Pins('L15', dir = 'i')),
			Subsignal('rx_data', Pins('P13 N13 P14 M15', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),

	]
	connectors = [
		Connector('j',  1, 'F3  F1  G3  - G2  H3  H5  F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  2, 'J4  K3  G1  - K4  C2  E3  F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  3, 'H4  K5  P1  - R1  L5  F2  F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  4, 'P4  R2  M8  - M9  T6  R6  F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  5, 'M11 N11 P12 - K15 N12 L16 F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  6, 'K16 J15 J16 - J12 H15 G16 F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  7, 'H13 J13 H12 - G14 H14 G15 F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  8, 'A15 F16 A14 - E13 B14 A13 F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j', 19, ' -  M13  -  - P11'),
	]

	@property
	def required_tools(self) -> list[str]:
		return super().required_tools + [
			'openFPGALoader'
		]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = dict(ecppack_opts = '--compress')
		overrides.update(kwargs)
		return super().toolchain_prepare(fragment, name, **overrides)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		tool = environ.get('OPENFPGALOADER', 'openFPGALoader')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([
				tool, '-c', 'ft232', '-m', bitstream_filename
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	Colorlight_5A75B_R70Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Attrs, Clock, Pins, Resource
from torii.build.run                     import BuildProducts
from torii.platform.resources            import DirectUSBResource, LEDResources, RGBLEDResource, SPIFlashResources
from torii.platform.vendor.lattice.ice40 import ICE40Platform

__all__ = (
	'FomuPVTPlatform',
)

class FomuPVTPlatform(ICE40Platform):
	device      = 'iCE40UP5K'
	package     = 'UWG30'
	default_clk = 'clk48'

	pretty_name = 'FOMU'
	description = 'Im Tomu FPGA Lattice iCE40 UP5K FPGA Board'

	resources   = [
		Resource(
			'clk48', 0, Pins('F4', dir = 'i'), Clock(48e6),
			Attrs(GLOBAL = True, IO_STANDARD = 'SB_LVCMOS')
		),

		*LEDResources(pins = 'A5', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		RGBLEDResource(0,
			r = 'B5', g = 'A5', b = 'C5', invert = True,
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),


		DirectUSBResource(0,
			d_p = 'A1', d_n = 'A2', pullup = 'A4',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),

		*SPIFlashResources(0,
			cs_n = 'C1', clk = 'D1', copi = 'F1', cipo = 'E1',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS'),
		),

		Resource('touch', 0, Pins('E4')),
		Resource('touch', 1, Pins('D5')),
		Resource('touch', 2, Pins('E5')),
		Resource('touch', 3, Pins('F5')),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		dfu_util = environ.get('DFU_UTIL', 'dfu-util')
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([dfu_util, '-D', bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	FomuPVTPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from itertools     import count, cycle

from torii.hdl.ast import Cat, Signal
from torii.hdl.dsl import Module
from torii.hdl.ir  import Elaboratable
from torii.build   import ResourceError

__all__ = (
	'Blinky',
)

class Blinky(Elaboratable):
	def elaborate(self, platform):
		m = Module()

		def get_all_resources(name):
			resources = []
			for number in count():
				try:
					resources.append(platform.request(name, number))
				except ResourceError:
					break
			return resources

		rgb_leds = [res for res in get_all_resources('rgb_led')]
		leds     = [res.o for res in get_all_resources('led')]
		leds.extend([led.r.o for led in rgb_leds])
		leds.extend([led.g.o for led in rgb_leds])
		leds.extend([led.b.o for led in rgb_leds])
		buttons  = [res.i for res in get_all_resources('button')]
		switches = [res.i for res in get_all_resources('switch')]

		inverts  = [0 for _ in leds]
		for index, button in zip(cycle(range(len(inverts))), buttons):
			inverts[index] ^= button
		for index, switch in zip(cycle(range(len(inverts))), switches):
			inverts[index] ^= switch

		clk_freq = platform.default_clk_frequency
		timer = Signal(range(int(clk_freq//2)), reset = int(clk_freq//2) - 1)
		flops = Signal(len(leds))

		m.d.comb += Cat(leds).eq(flops ^ Cat(inverts))
		with m.If(timer == 0):
			m.d.sync += timer.eq(timer.reset)
			m.d.sync += flops.eq(~flops)
		with m.Else():
			m.d.sync += timer.eq(timer - 1)

		return m



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildPlan, BuildProducts
from torii.hdl.ir                       import Fragment
from torii.platform.resources           import (
	ButtonResources, DirectUSBResource, LEDResources, SDCardResources, SPIFlashResources
)
from torii.platform.vendor.lattice.ecp5 import ECP5Platform

__all__ = (
	'LogicbonePlatform',
	'Logicbone85FPlatform',
)

class LogicbonePlatform(ECP5Platform):
	name        = 'Logicbone'
	device      = 'LFE5UM5G-45F'
	package     = 'BG381'
	speed       = '8'
	default_clk = 'refclk'

	pretty_name = 'Logicbone'
	description = 'Logicbone Lattice ECP5-5G-45F Development Board'

	resources   = [
		Resource(
			'refclk', 0, Pins('M19', dir = 'i'), Clock(25e6), Attrs(IO_TYPE = 'LVCMOS18')
		),

		# SerDes connections.
		Resource('serdes', 0,
			Subsignal('rx', DiffPairs('Y5', 'Y6')),
			Subsignal('tx', DiffPairs('W4', 'W5')),
		),
		Resource('serdes', 1,
			Subsignal('rx', DiffPairs('Y8', 'Y7')),
			Subsignal('tx', DiffPairs('W8', 'W9')),
		),

		# VBUS Detection is on the USB-C PD controller for both ports.
		DirectUSBResource('usb', 0,
			d_p = 'B12', d_n = 'C12', pullup = 'C16',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		DirectUSBResource('usb', 1,
			d_p = 'R1',  d_n = 'T1',  pullup = 'T2',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),

		*LEDResources(pins = 'D16 C15 C13 B13', attrs = Attrs(IO_TYPE = 'LVCMOS33'), invert = True),

		*ButtonResources(pins = 'U2', attrs = Attrs(IO_TYPE = 'LVCMOS33'), invert = True),

		*SPIFlashResources(0,
			cs_n = 'R2', clk = 'U3', cipo = 'V2', copi = 'W2', wp_n = 'Y2', hold_n = 'W1',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),

		*SDCardResources(0,
			clk = 'E11', cmd = 'D15', cd = 'D14',
			dat0 = 'D13', dat1 = 'E13', dat2 = 'E15', dat3 = 'E13',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),

		Resource(
			'eth_clk125', 0, Pins('A19', dir = 'i'), Clock(125e6), Attrs(IO_TYPE = 'LVCMOS33')
		),
		Resource('eth_rgmii', 0,
			# Stolen for sys_reset usage on prototypes.
			# Subsignal('rst',     PinsN('U17', dir = 'o')),
			Subsignal('int',     Pins('B20', dir = 'i')),
			Subsignal('mdc',     Pins('D12', dir = 'o')),
			Subsignal('mdio',    Pins('B19', dir = 'io')),
			Subsignal('tx_clk',  Pins('A15', dir = 'o')),
			Subsignal('tx_ctl',  Pins('B15', dir = 'o')),
			Subsignal('tx_data', Pins('A12 A13 C14 A14', dir = 'o')),
			Subsignal('rx_clk',  Pins('B18', dir = 'i')),
			Subsignal('rx_ctl',  Pins('A18', dir = 'i')),
			Subsignal('rx_data', Pins('B17 A17 B16 A16', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),

		Resource('ddr3', 0,
			Subsignal('rst',     PinsN('P1', dir = 'o')),
			Subsignal('clk',     DiffPairs('M4', 'N5', dir = 'o'), Attrs(IO_TYPE = 'LVDS')),
			Subsignal('clk_en',  Pins('K4', dir = 'o')),
			Subsignal('cs',      PinsN('M3', dir = 'o')),
			Subsignal('we',      PinsN('E4', dir = 'o')),
			Subsignal('ras',     PinsN('L1', dir = 'o')),
			Subsignal('cas',     PinsN('M1', dir = 'o')),
			Subsignal('a',       Pins('D5 F4 B3 F3 E5 C3 C4 A5 A3 B5 G3 F5 D2 A4 D3 E3', dir = 'o')),
			Subsignal('ba',      Pins('B4 H5 N2', dir = 'o')),
			Subsignal('dqs',     DiffPairs('K2 H4', 'J1 G5', dir = 'io'), Attrs(IO_TYPE = 'LVDS')),
			Subsignal('dq',      Pins('G2 K1 F1 K3 H2 J3 G1 H1 B1 E1 A2 F2 C1 E2 C2 D1', dir = 'io')),
			Subsignal('dm',      Pins('L4 J5', dir = 'o')),
			Subsignal('odt',     Pins('C5', dir = 'o')),
			Attrs(IO_TYPE = 'SSTL135_I')
		)
	]
	connectors = [
		Connector('P8', 0, '''
		-   -   C20 D19 D20 E19 E20 F19 F20 G20 -   -   -   -   -   -
		-   -   -   -   -   -   G19 H20 J20 K20 C18 D17 D18 E17 E18 F18
		F17 G18 E16 F16 G16 H16 J17 J16 H18 H17 J19 K19 J18 K18
		'''),
		Connector('P9', 0, '''
		-   -   -   -   -   -   -   -   -   -   -   A11 B11 A10 C10 A9
		B9  C11 A8  -   -   D9  C8  B8  A7  A6  B6  D8  C7  D7  C6  D6
		-   -   -   -   -   -   -   -   -   B10 E10 -   -   -
		'''),
	]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = dict(ecppack_opts = '--compress --spimode qspi --freq 38.8')
		overrides.update(kwargs)
		return super().toolchain_prepare(fragment, name, **overrides)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call
		dfu_util = environ.get('DFU_UTIL', 'dfu-util')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([
				dfu_util, '-d', '1d50:615d', '-a', '0', '-R',
				'-D', bitstream_filename
			])


class Logicbone85FPlatform(LogicbonePlatform):
	name        = 'Logicbone (85F Variant)'
	device      = 'LFE5UM5G-85F'

if __name__ == '__main__':
	import argparse

	from ..test.blinky import Blinky

	parser = argparse.ArgumentParser()
	parser.add_argument(
		'--variant',
		choices = ('85', '45'),
		default = '45',
		help = 'platform variant (default: %(default)s)'
	)

	args = parser.parse_args()

	if args.variant == '45':
		platform = LogicbonePlatform()

	if args.variant == '85':
		platform = Logicbone85FPlatform()

	platform.build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import Attrs, Clock, DiffPairs, Resource
from torii.build.run              import BuildProducts
from torii.platform.resources     import LEDResources, UARTResource
from torii.platform.vendor.xilinx import XilinxPlatform

__all__ = (
	'KC705Platform',
)

class KC705Platform(XilinxPlatform):
	device      = 'xc7k325t'
	package     = 'ffg900'
	speed       = '2'
	default_clk = 'clk156'

	pretty_name = 'KC705'
	description = 'Xilinx KC705 Kintex7-325T Evaluation Board'

	resources   = [
		Resource(
			'clk156', 0, DiffPairs('K28', 'K29', dir = 'i'), Clock(156e6), Attrs(IOSTANDARD = 'LVDS_25')
		),

		*LEDResources(
			pins = 'AB8 AA8 AC9 AB9 AE26 G19 E18 F16',
			attrs = Attrs(IOSTANDARD = 'LVCMOS15')
		),

		UARTResource(0,
			rx = 'M19', tx = 'K24',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		openocd = environ.get('OPENOCD', 'openocd')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([openocd,
				'-c', f'source [find board/kc705.cfg]; init; pld load 0 {bitstream_filename}; exit'
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	KC705Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from .versa_ecp5 import VersaECP5Platform

__all__ = (
	'VersaECP55GPlatform',
)

class VersaECP55GPlatform(VersaECP5Platform):
	device      = 'LFE5UM5G-45F'

	pretty_name = 'Versa 5G'
	description = 'Lattice Versa 5G ECP5-5G-45F Evaluation Board'

	# Everything else is identical between 3G and 5G Versa boards.


if __name__ == '__main__':
	from ..test.blinky import Blinky
	VersaECP55GPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import Attrs, Clock, Connector, Pins, Resource, Subsignal
from torii.build.run              import BuildProducts
from torii.platform.resources     import (
	ButtonResources, Display7SegResource, LEDResources, NORFlashResources, PS2Resource, SDCardResources, SDRAMResource,
	SwitchResources, UARTResource, VGAResource
)
from torii.platform.vendor.altera import AlteraPlatform

__all__ = (
	'DE0Platform',
)

class DE0Platform(AlteraPlatform):
	device      = 'EP3C16' # Cyclone III 15K LEs
	package     = 'F484'   # FBGA-484
	speed       = 'C6'
	default_clk = 'clk50'

	pretty_name = 'DE0'
	description = 'terasIC DE0 Altera Cyclone III Development Board'

	resources   = [
		Resource(
			'clk50', 0, Pins('G21', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 1, Pins('B12', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),

		*LEDResources(
			pins = 'J1 J2 J3 H1 F2 E1 C1 C2 B2 B1',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*ButtonResources(
			pins = 'H2 G3 F1', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*SwitchResources(
			pins = 'J6 H5 H6 G4 G5 J7 H7 E3 E4 D2',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(0,
			a = 'E11', b = 'F11', c = 'H12', d = 'H13', e = 'G12', f = 'F12', g = 'F13', dp = 'D13', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(1,
			a = 'A13', b = 'B13', c = 'C13', d = 'A14', e = 'B14', f = 'E14', g = 'A15', dp = 'B15', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(2,
			a = 'D15', b = 'A16', c = 'B16', d = 'E15', e = 'A17', f = 'B17', g = 'F14', dp = 'A18', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(3,
			a = 'B18', b = 'F15', c = 'A19', d = 'B19', e = 'C19', f = 'D19', g = 'G15', dp = 'G16', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		UARTResource(0,
			rx = 'U22', tx = 'U21', rts = 'V22', cts = 'V21',
			attrs = Attrs(io_standard = '3.3-V LVTTL'),
			role = 'dce'
		),

		Resource('display_hd44780', 0,
			Subsignal('e', Pins('E21', dir = 'o')),
			Subsignal('d', Pins('D22 D21 C22 C21 B22 B21 D20 C20', dir = 'io')),
			Subsignal('rw', Pins('E22', dir = 'o')),
			Subsignal('rs', Pins('F22', dir = 'o')),
			# Backlight
			Subsignal('bl', Pins('F21', dir = 'o')),
			Attrs(io_standard = '3.3-V LVTTL')
		),

		VGAResource(0,
			r = 'H19 H17 H20 H21',
			g = 'H22 J17 K17 J21',
			b = 'K22 K21 J22 K18',
			hs = 'L21', vs = 'L22',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		PS2Resource(0, # Keyboard
			clk = 'P22', dat = 'P21', attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		PS2Resource(1, # Mouse
			clk = 'R21', dat = 'R22', attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		*SDCardResources(0,
			clk = 'Y21', cmd = 'Y22', dat0 = 'AA22', dat3 = 'W21', wp_n = 'W20',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		SDRAMResource(0,
			clk = 'E5', cke = 'E6', cs_n = 'G7', we_n = 'D6', ras_n = 'F7', cas_n = 'G8',
			ba = 'B5 A4', a = 'C4 A3 B3 C3 A5 C6 B6 A6 C7 B7 B4 A7 C8',
			dq = 'D10 G10 H10 E9 F9 G9 H9 F8 A8 B9 A9 C10 B10 A10 E10 F10', dqm = 'E7 B8',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		*NORFlashResources(0,
			rst = 'R1', byte_n = 'AA1',
			cs_n = 'N8', oe_n = 'R6', we_n = 'P4', wp_n = 'T3', by = 'M7',
			a = 'P7 P5 P6 N7 N5 N6 M8 M4 P2 N2 N1 M3 M2 M1 L7 L6 AA2 M5 M6 P1 P3 R2',
			dq = 'R7 P8 R8 U1 V2 V3 W1 Y1 T5 T7 T4 U2 V1 V4 W2 Y2',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
	]
	connectors  = [
		Connector('j', 4,
			'AB12 AB16 AA12 AA16 AA15 AB15 AA14 AB14 AB13 AA13 -    -    '
			'AB10 AA10 AB8  AA8  AB5  AA5  AB3  AB4  AA3  AA4  V14  U14  '
			'Y13  W13  U13  V12  -    -    R10  V11  Y10  W10  T8   V8   '
			'W7   W6   V5   U7   -    -    -    -    -    -    -    -    '
		),
		Connector('j', 5,
			'AB11 AA20 AA11 AB20 AA19 AB19 AB18 AA18 AA17 AB17 -    -    '
			'Y17  W17  U15  T15  W15  V15  R16  AB9  T16  AA9  AA7  AB7  '
			'T14  R14  U12  T12  -    -    R11  R12  U10  T10  U9   T9   '
			'Y7   U8   V6   V7   -    -    -    -    -    -    -    -    '
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	DE0Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Attrs, Clock, Connector, Pins, Resource
from torii.build.run                     import BuildProducts
from torii.platform.resources            import (
	ButtonResources, LEDResources, SRAMResource, SwitchResources, UARTResource
)
from torii.platform.vendor.lattice.ice40 import ICE40Platform

__all__ = (
	'BlackIceIIPlatform',
)

class BlackIceIIPlatform(ICE40Platform):
	device      = 'iCE40HX4K'
	package     = 'TQ144'
	default_clk = 'clk100'

	pretty_name = 'BlackIce II'
	description = 'myStorm BlackIce II Lattice iCE40-HX4k Development Board'

	resources   = [
		Resource('clk100', 0, Pins('129', dir = 'i'),
			Clock(100e6), Attrs(GLOBAL = True, IO_STANDARD = 'SB_LVCMOS')
		),

		*LEDResources(pins = '71 67 68 70', attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		# Color aliases
		Resource('led_b', 0, Pins('71', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_g', 0, Pins('67', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_o', 0, Pins('68', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_r', 0, Pins('70', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),

		*ButtonResources(pins = '63 64',       invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		*SwitchResources(pins = '37 38 39 41', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),

		UARTResource(0,
			rx = '88', tx = '85', rts = '91', cts = '94',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS', PULLUP = 1),
			role = 'dce'
		),

		SRAMResource(0,
			cs_n = '23', oe_n = '29', we_n = '120',
			a = '137 138 139 141 142 42 43 44 73 74 75 76 115 116 117 118 119 78',
			d = '136 135 134 130 125 124 122 121 62 61 60 56 55 48 47 45',
			dm_n = '24 28',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS'),
		),
	]
	connectors  = [
		Connector('pmod', 0, ' 94  91  88  85 - -  95  93  90  87 - -'),  # PMOD1/2
		Connector('pmod', 1, '105 102  99  97 - - 104 101  98  96 - -'),  # PMOD3/4
		Connector('pmod', 2, '143 114 112 107 - - 144 113 110 106 - -'),  # PMOD5/6
		Connector('pmod', 3, ' 10   9   2   1 - -   8   7   4   3 - -'),  # PMOD7/8
		Connector('pmod', 4, ' 20  19  16  15 - -  18  17  12  11 - -'),  # PMOD9/10
		Connector('pmod', 5, ' 34  33  22  21 - -  32  31  26  25 - -'),  # PMOD11/12
		Connector('pmod', 6, ' 37  38  39  41 - -'),  # PMOD13
		Connector('pmod', 7, ' 71  67  68  70 - -'),  # PMOD14
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from subprocess import check_call

		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([
				'cp', bitstream_filename, '/dev/ttyACM0'
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	BlackIceIIPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Attrs, Clock, Connector, Pins, PinsN, Resource
from torii.build.run                     import BuildProducts
from torii.platform.resources            import (
	ButtonResources, DirectUSBResource, LEDResources, RGBLEDResource, SPIFlashResources
)
from torii.platform.vendor.lattice.ice40 import ICE40Platform

__all__ = (
	'ICEBreakerBitsyPlatform',
)

class ICEBreakerBitsyPlatform(ICE40Platform):
	device      = 'iCE40UP5K'
	package     = 'SG48'
	default_clk = 'clk12'

	pretty_name = 'iCEBreaker Bitsy'
	description = '1BitSquared iCEBreaker Bitsy Lattice iCE40-UP5K Development Board'

	resources   = [
		Resource(
			'clk12', 0, Pins('35', dir = 'i'), Clock(12e6),
			Attrs(GLOBAL = True, IO_STANDARD = 'SB_LVCMOS')
		),

		DirectUSBResource(0,
			d_p = '42', d_n = '38', pullup = '37',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),

		*SPIFlashResources(0,
			cs_n = '16', clk = '15', copi = '14', cipo = '17', wp_n = '18', hold_n = '19',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),

		RGBLEDResource(0,
			r = '39', g = '40', b = '41', invert = True,
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		*LEDResources(pins = '25 6', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_r', 0, PinsN('25', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_g', 0, PinsN('6', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),

		*ButtonResources(pins = '2', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
	]
	connectors = [
		Connector('edge', 0,  # Pins bottom P0 - P12,
			{
				'0' : '47', '1' : '44', '2' : '48', '3' : '45', '4' :  '4', '5' :  '3',
				'6' :  '9', '7' : '10', '8' : '11', '9' : '12', '10': '21', '11': '13',
				'12': '20', '13': '25', '14': '23', '15': '27', '16': '26', '17': '28',
				'18': '31', '19': '32', '20': '34', '21': '36', '22': '43', '23': '46'
			}
		),
		Connector('pmod', 1, ' 0  2  4  6 - -  1  3  5  7 - -', conn = ('edge', '0')), # PMOD 1
		Connector('pmod', 2, '22 19 16 17 - - 21 18 15 20 - -', conn = ('edge', '0')), # PMOD 2
		Connector('pmod', 3, '14  9 11  8 - - 13 10 12 23 - -', conn = ('edge', '0'))  # PMOD 3
	]

	def toolchain_program(
		self, products: BuildProducts, name: str,
		run_vid: str | None = None, run_pid: str | None = None,
		dfu_vid: str = '1d50', dfu_pid: str = '6146', reset: bool = True
	) -> None:
		from os         import environ
		from subprocess import check_call

		dfu_util = environ.get('DFU_UTIL', 'dfu-util')

		# Construct the device runtime and DFU vid pid string
		dev_str = ''
		if run_vid or run_pid:
			dev_str = f'{run_vid or ""}:{run_pid or ""}'
		dev_str += f',{dfu_vid or ""}:{dfu_pid or ""}'

		# Construct the argument list for dfu-util
		args = [dfu_util, '-d', dev_str, '-a', '0']
		if reset:
			args.append('-R')
		args.append('-D')

		# Run dfu-util
		with products.extract(f'{name}.bin') as bitstream_filename:
			args.append(bitstream_filename)
			check_call(args)


if __name__ == '__main__':
	from ..test.blinky import Blinky
	ICEBreakerBitsyPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from pathlib                  import Path
from unittest                 import TestCase

from torii.platform.vendor    import LatticeICE40Platform

from torii_boards.lattice     import (
	ice40_hx1k_blink_evn, ice40_hx8k_b_evn, ice40_up5k_b_evn, icebreaker, icebreaker_bitsy
)
from torii_boards.test.blinky import Blinky

BUILD_DIR = (Path(__file__).parent / '..' / 'build' / 'test_boards').resolve()

class ToriiBoardsICE40Tests(TestCase):
	def _test_platform(platform: LatticeICE40Platform):
		BUILD_DIR.mkdir(exist_ok = True, parents = True)

		def test_platform(self):
			plat_name  = type(platform).__name__
			plat_build = (BUILD_DIR / plat_name)
			platform.build(Blinky(), plat_name, plat_build, do_build = True, do_program = False)
		return test_platform

	test_icebreaker           = _test_platform(icebreaker.ICEBreakerPlatform())
	test_icebreaker_bitsy     = _test_platform(icebreaker_bitsy.ICEBreakerBitsyPlatform())
	test_ice40_up5k_b_evn     = _test_platform(ice40_up5k_b_evn.ICE40UP5KBEVNPlatform())
	test_ice40_hx1k_blink_evn = _test_platform(ice40_hx1k_blink_evn.ICE40HX1KBlinkEVNPlatform())
	test_ice40_hx8k_b_evn     = _test_platform(ice40_hx8k_b_evn.ICE40HX8KBEVNPlatform())



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import Attrs, Clock, Pins, Resource
from torii.build.run              import BuildProducts
from torii.platform.resources     import LEDResources, UARTResource
from torii.platform.vendor.xilinx import XilinxPlatform

__all__ = (
	'EBAZ4205Platform',
)

class EBAZ4205Platform(XilinxPlatform):
	device      = 'xc7z010'
	package     = 'clg400'
	speed       = '1'
	default_clk = 'clk33_333'

	pretty_name = 'EBAZ4205'
	description = 'EBAZ4205 Ebit E9+ Xilinx Zynq-7010 Board'

	resources   = [
		Resource(
			'clk33_333', 0, Pins('N18', dir = 'i'), Clock(33.333e6), Attrs(IOSTANDARD = 'LVCMOS33')
		),

		*LEDResources(
			pins = 'W14 W13',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		UARTResource(0,
			rx = 'B19', tx = 'B20',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
	]

	connectors = [
	]

	def toolchain_program(self, products: BuildProducts, name: str, **kwargs) -> None:
		from os         import environ
		from subprocess import run

		xc3sprog = environ.get('XC3SPROG', 'xc3sprog')
		with products.extract(f'{name}.bit') as bitstream_filename:
			run([xc3sprog, '-c', 'jtaghs1_fast', '-p', '1', bitstream_filename], check = True)


if __name__ == '__main__':
	from ..test.blinky import Blinky
	EBAZ4205Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.platform.resources     import I2CResource, LEDResources, SDCardResources, UARTResource, VGADACResource
from torii.platform.vendor.xilinx import XilinxPlatform

__all__ = (
	'Mega65r3Platform',
)

class Mega65r3Platform(XilinxPlatform):
	device = 'xc7a200t'
	package = 'fbg484'
	speed = '2'
	default_clk = 'clk100'

	pretty_name = 'Mega65'
	description = 'Mega65 Xilinx Artix7-200T Based 8-bit computer'

	resources = [
		Resource('clk100', 0, Pins('V13', dir = 'i'), Clock(100e6), Attrs(IOSTANDARD = 'LVCMOS33')),

		*LEDResources(pins = 'U22', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),

		Resource('iec', 0,
			Subsignal('rst', Pins('AB21', dir = 'o')),
			Subsignal('atn', Pins('N17', dir = 'o')),

			Subsignal('data_en', Pins('Y21', dir = 'o')),
			Subsignal('data_o', Pins('Y22', dir = 'o')),
			Subsignal('data_i', Pins('AB22', dir = 'i'), Attrs(PULLUP = 'TRUE')),

			Subsignal('clk_en', Pins('AA21', dir = 'o')),
			Subsignal('clk_o', Pins('Y19', dir = 'o')),
			Subsignal('clk_i', Pins('Y18', dir = 'i'), Attrs(PULLUP = 'TRUE')),

			Subsignal('srq_en', Pins('AB20', dir = 'o')),
			Subsignal('srq_o', Pins('U20', dir = 'o')),
			Subsignal('srq_i', Pins('AA18', dir = 'i')),

			Attrs(IOSTANDARD = 'LVCMOS33'),
		),

		Resource('cart', 0,
			Subsignal('ctrl_en', Pins('G18', dir = 'o')),
			Subsignal('ctrl_dir', Pins('U17', dir = 'o')),

			Subsignal('addr_en', Pins('L19', dir = 'o')),
			Subsignal('addr_dir_lo', Pins('L21', dir = 'o')),
			Subsignal('addr_dir_hi', Pins('L18', dir = 'o')),

			Subsignal('data_en', Pins('U21', dir = 'o')),
			Subsignal('data_dir', Pins('V22', dir = 'o')),

			Subsignal('phi2', Pins('V17', dir = 'o')),
			Subsignal('dotclk', Pins('AA19', dir = 'o')),
			Subsignal('rst', Pins('N14', dir = 'o')),

			Subsignal('nmi', Pins('W17', dir = 'i')),
			Subsignal('irq', Pins('P14', dir = 'i')),
			Subsignal('dma', Pins('P15', dir = 'i')),

			Subsignal('exrom', Pins('R19', dir = 'io')),
			Subsignal('ba', Pins('N13', dir = 'io')),
			Subsignal('rw', Pins('R18', dir = 'io')),
			Subsignal('rom_lo', Pins('AB18', dir = 'io')),
			Subsignal('rom_hi', Pins('T18', dir = 'io')),
			Subsignal('io1', Pins('N15', dir = 'io')),
			Subsignal('game', Pins('W22', dir = 'io')),
			Subsignal('io2', Pins('AA20', dir = 'io')),

			Subsignal('data', Pins('P16 R17 P20 R16 U18 V18 W20 W21', dir = 'io')),

			Subsignal('addr', Pins(
				'K19 K18 K21 M22 L20 J20 J21 K22 '
				'H17 H20 G20 J15 H19 M20 N22 H18',
				dir = 'io'
			)),

			Attrs(IOSTANDARD = 'LVCMOS33'),
		),

		Resource('keyboard', 0,
			Subsignal('clk', Pins('A14', dir = 'o')),
			Subsignal('out', Pins('A13', dir = 'o')),
			Subsignal('in', Pins('C13', dir = 'i')),

			Subsignal('tck', Pins('E13', dir = 'o')),
			Subsignal('tdo', Pins('E14', dir = 'o')),
			Subsignal('tdi', Pins('D15', dir = 'i')),
			Subsignal('tms', Pins('D14', dir = 'o')),
			Subsignal('jtag_en', Pins('B13', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),

		Resource('paddle', 0,
			Subsignal('in', Pins('H13 G15 J14 J22', dir = 'i')),
			Subsignal('drain', Pins('H22', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),

		Resource('joystick', 0,
			Subsignal('up', Pins('C14', dir = 'i')),
			Subsignal('down', Pins('F16', dir = 'i')),
			Subsignal('left', Pins('F14', dir = 'i')),
			Subsignal('right', Pins('F13', dir = 'i')),
			Subsignal('fire', Pins('E17', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),

		Resource('joystick', 1,
			Subsignal('up', Pins('W19', dir = 'i')),
			Subsignal('down', Pins('P17', dir = 'i')),
			Subsignal('left', Pins('F21', dir = 'i')),
			Subsignal('right', Pins('C15', dir = 'i')),
			Subsignal('fire', Pins('F15', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),

		VGADACResource(0,
			clk = 'AA9',
			r = 'U15 V15 T14  Y17  Y16  AB17 AA16 AB16',
			g = 'Y14 W14 AA15 AB15 Y13  AA14 AA13 AB13',
			b = 'W10 Y12 AB12 AA11 AB11 Y11  AB10 AA10',
			hs = 'W12', vs = 'V14',
			extras = [
				Subsignal('sync', PinsN('V10', dir = 'o')),
				Subsignal('blank', PinsN('W11', dir = 'o')),
			],
			attrs = Attrs(IOSTANDARD = 'LVCMOS33'),
		),

		Resource('hdmi', 0,
			Subsignal('clk', DiffPairs('W1', 'Y1', dir = 'o')),
			Subsignal('data', DiffPairs('AA1 AB3 AA5', 'AB1 AB2 AB5', dir = 'o')),
			Subsignal('scl', Pins('AB7', dir = 'io')),
			Subsignal('sda', Pins('V9', dir = 'io')),
			Subsignal('en', Pins('AB8', dir = 'o')), # aka ls_oe
			Subsignal('hpd', Pins('Y8', dir = 'i')),
			Subsignal('hpd_en', Pins('M15', dir = 'o')),
			Subsignal('cec', Pins('W9', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),

		I2CResource(0, scl = 'A15', sda = 'A16', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),

		# Seeed Grove
		I2CResource('grove', 0, scl = 'G21', sda = 'G22', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),

		Resource('audio', 0,
			Subsignal('left', Pins('L6', dir = 'o')),
			Subsignal('right', Pins('F4', dir = 'o')),
			Subsignal('sd', Pins('F18', dir = 'o')),
			Subsignal('speaker', Pins('E16', dir = 'o')),
			Subsignal('mclk', Pins('D16', dir = 'o')),
			Subsignal('bclk', Pins('E19', dir = 'o')),
			Subsignal('sync', Pins('F19', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),

		Resource('spi_flash', 0,
			Subsignal('data', Pins('P22 R22 P21 R21', dir = 'io'), Attrs(PULLUP = 'TRUE')),
			Subsignal('cs', PinsN('T19', dir = 'o')),
		),

		Resource('hyper_ram', 0,
			Subsignal('clk', Pins('D22', dir = 'o'), Attrs(SLEW = 'FAST', DRIVE = '16')),
			Subsignal('data', Pins('A21 D21 C20 A20 B20 A19 E21 E22', dir = 'io'), Attrs(SLEW = 'FAST', DRIVE = '16')),
			Subsignal('rwds', Pins('B21', dir = 'io'), Attrs(SLEW = 'FAST', DRIVE = '16')),
			Subsignal('rst', Pins('B22', dir = 'o')),
			Subsignal('cs', Pins('C22', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33', PULLUP = 'FALSE'),
		),

		Resource('ethernet', 0,
			Subsignal('led', Pins('R14', dir = 'o')),
			Subsignal('clk', Pins('L4', dir = 'o'), Attrs(SLEW = 'FAST')),
			Subsignal('rst', Pins('K6', dir = 'o')),
			Subsignal('mdio', Pins('L5', dir = 'io')),
			Subsignal('mdc', Pins('J6', dir = 'o')),
			Subsignal('rxd', Pins('P4 L1', dir = 'i')),
			Subsignal('txd', Pins('L3 K3', dir = 'o'), Attrs(SLEW = 'SLOW', DRIVE = '4')),
			Subsignal('rxer', Pins('M6', dir = 'i')),
			Subsignal('txen', Pins('J4', dir = 'o'), Attrs(SLEW = 'SLOW', DRIVE = '4')),
			Subsignal('rxdv', Pins('K4', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),

		UARTResource(0, rx = 'L14', tx = 'L13', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),

		Resource('max10', 0,
			Subsignal('tx', Pins('M13', dir = 'i')),
			Subsignal('rx', Pins('K16', dir = 'o')),
			Subsignal('rst', Pins('L16', dir = 'io')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),

		*SDCardResources(0,
			clk = 'B17', cmd = 'B16', cd = 'D17', wp_n = 'C17',
			dat0 = 'B18', dat1 = 'C18', dat2 = 'C19', dat3 = 'B15',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33'),
		),

		*SDCardResources(1,
			clk = 'G2', cmd = 'J2', cd = 'K1', wp_n = None,
			dat0 = 'H2', dat1 = 'H3', dat2 = 'J1', dat3 = 'K2',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33'),
		),

		Resource('floppy', 0,
			Subsignal('density', Pins('P6', dir = 'o')),
			Subsignal('motor', Pins('M5 H15', dir = 'o')),
			Subsignal('select', Pins('N5 G17', dir = 'o')),
			Subsignal('step_dir', Pins('P5', dir = 'o')),
			Subsignal('step', Pins('M3', dir = 'o')),
			Subsignal('wdata', Pins('N4', dir = 'o')),
			Subsignal('wgate', Pins('N3', dir = 'o')),
			Subsignal('side1', Pins('M1', dir = 'o')),
			Subsignal('index', Pins('M2', dir = 'i')),
			Subsignal('track0', Pins('N2', dir = 'i')),
			Subsignal('wp', Pins('P2', dir = 'i')),
			Subsignal('rdata', Pins('P1', dir = 'i')),
			Subsignal('diskchanged', Pins('R1', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),
	]

	connectors = [
		Connector('pmod', 0, 'G1 E1 C2 B1 - - F1 D1 B2 A1 - -'),
		Connector('pmod', 1, 'E2 D2 G4 J5 - - F3 E3 H4 H5 - -'),
		Connector('test', 0, 'T16 U16 W16 J19 K17 N19 N20 D20'),
	]


if __name__ == '__main__':
	from ..test.blinky import Blinky
	Mega65r3Platform(toolchain = 'yosys_nextpnr').build(
		Blinky(), do_build = True, do_program = False)



# SPDX-License-Identifier: BSD-2-Clause

import subprocess

from torii.build                  import Attrs, Clock, Connector, Pins, Resource
from torii.platform.resources     import (
	ButtonResources, LEDResources, RGBLEDResource, SPIFlashResources, UARTResource
)
from torii.platform.vendor.xilinx import XilinxPlatform

'''
Example Usage:
	platform = CModS7_Platform(toolchain = 'Symbiflow')
	platform.build(Top(), do_program = True)

Supported programmer:
	openFPGALoader
'''

__all__ = (
	'CmodS7_Platform',
)

class CmodS7_Platform(XilinxPlatform):
	device      = 'xc7s25'
	package     = 'csga225'
	speed       = '1'
	default_clk = 'clk12'

	pretty_name = 'Cmod S7'
	description = 'Digilent Cmod S7 Xilinx Spartan7-25 FPGA Module'

	resources   = [
		Resource(
			'clk12', 0, Pins('M9', dir = 'i'), Clock(12e6), Attrs(IOSTANDARD = 'LVCMOS33')
		),

		*LEDResources(pins = 'E2 K1 J1 E1', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),

		RGBLEDResource(0,
			r = 'F2', g = 'D3', b = 'F1', invert = True,
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		*ButtonResources(pins = 'D2 D1', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),

		UARTResource(0, rx = 'L12', tx = 'K15', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),

		# Clock only via STARTUPE2 primitive
		*SPIFlashResources(0,
			cs_n = 'L11', clk = 'F5', copi = 'H14', cipo = 'H15', wp_n = 'J12',
			hold_n = 'K13',
			attrs = Attrs(IOSTANDARD='LVCMOS33')
		),

		# One-wire interface to crypto authentication device
		# May not be populated on the board
		Resource('atsha204a', 0, Pins('D17', dir = 'io'), Attrs(IOSTANDARD = 'LVCMOS33'))
	]
	connectors  = [
		Connector('pmod', 0, 'J2 H2 H4 F3 - - H3 H1 G1 F4 - -'), # JA

		# Pin 24/25 are VCC and GND
		# Pin 32/33 are analog (XADC)
		# Pin 9-15 and 34-39 do not exist
		Connector('gpio', 0,
			'L1  M4  M3  N2  M2  P3  N3  P1  N1  -   -   -  '
			'-   -   -   P14 P15 N13 N15 N14 M15 M14 L15 -  '
			'-   L14 K14 J15 L13 M13 J11 -   -   -   -   -  '
			'-   -   -   C5  A2  B2  B1  C1  B3  B4  A3  A4 '
		),

		Connector('xadc', 0, {
			'vaux5_n':  'A13',
			'vaux5_p':  'A14',
			'vaux12_n': 'A11',
			'vaux12_p': 'A12'
		})
	]

	def toolchain_program(self, products, name):
		with products.extract(f'{name}.bit') as bitstream_filename:
			subprocess.check_call([
				'openFPGALoader', '-c', 'digilent', '--fpga-part', self.device, bitstream_filename
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	CmodS7_Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildProducts
from torii.platform.resources           import LEDResources, SPIFlashResources, SwitchResources, UARTResource
from torii.platform.vendor.lattice.ecp5 import ECP5Platform

__all__ = (
	'VersaECP5Platform',
)

class VersaECP5Platform(ECP5Platform):
	device      = 'LFE5UM-45F'
	package     = 'BG381'
	speed       = '8'
	default_clk = 'clk100'
	default_rst = 'rst'

	pretty_name = 'Versa'
	description = 'Lattice Versa ECP5-45F Evaluation Board'

	resources   = [
		Resource('rst', 0, PinsN('T1', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33')),
		Resource('clk100', 0, DiffPairs('P3', 'P4', dir = 'i'), Clock(100e6), Attrs(IO_TYPE = 'LVDS')),
		Resource('pclk', 0, DiffPairs('A4', 'A5', dir = 'i'), Attrs(IO_TYPE = 'LVDS')),

		*LEDResources(
			pins = 'E16 D17 D18 E18 F17 F18 E17 F16', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS25')
		),

		Resource('alnum_led', 0,
			Subsignal('a', PinsN('M20', dir = 'o')),
			Subsignal('b', PinsN('L18', dir = 'o')),
			Subsignal('c', PinsN('M19', dir = 'o')),
			Subsignal('d', PinsN('L16', dir = 'o')),
			Subsignal('e', PinsN('L17', dir = 'o')),
			Subsignal('f', PinsN('M18', dir = 'o')),
			Subsignal('g', PinsN('N16', dir = 'o')),
			Subsignal('h', PinsN('M17', dir = 'o')),
			Subsignal('j', PinsN('N18', dir = 'o')),
			Subsignal('k', PinsN('P17', dir = 'o')),
			Subsignal('l', PinsN('N17', dir = 'o')),
			Subsignal('m', PinsN('P16', dir = 'o')),
			Subsignal('n', PinsN('R16', dir = 'o')),
			Subsignal('p', PinsN('R17', dir = 'o')),
			Subsignal('dp', PinsN('U1', dir = 'o')),
			Attrs(IO_TYPE = 'LVCMOS25')
		),

		*SwitchResources(
			pins = {0: 'H2',  1: 'K3',  2: 'G3',  3: 'F2' },
			attrs = Attrs(IO_TYPE = 'LVCMOS15')
		),
		*SwitchResources(
			pins = {4: 'J18', 5: 'K18', 6: 'K19', 7: 'K20'},
			attrs = Attrs(IO_TYPE = 'LVCMOS25')
		),

		UARTResource(0,
			rx = 'C11', tx = 'A11',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),

		*SPIFlashResources(0,
			cs_n = 'R2', clk = 'U3', cipo = 'W2', copi = 'V2', wp_n = 'Y2', hold_n = 'W1',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),

		Resource(
			'eth_clk125', 0, Pins('L19', dir = 'i'), Clock(125e6), Attrs(IO_TYPE = 'LVCMOS25')
		),
		Resource(
			'eth_clk125_pll', 0, Pins('U16', dir = 'i'), Clock(125e6), Attrs(IO_TYPE = 'LVCMOS25')
		), # NC by default
		Resource('eth_rgmii', 0,
			Subsignal('rst',     PinsN('U17', dir = 'o')),
			Subsignal('mdc',     Pins('T18', dir = 'o')),
			Subsignal('mdio',    Pins('U18', dir = 'io')),
			Subsignal('tx_clk',  Pins('P19', dir = 'o')),
			Subsignal('tx_ctl',  Pins('R20', dir = 'o')),
			Subsignal('tx_data', Pins('N19 N20 P18 P20', dir = 'o')),
			Subsignal('rx_clk',  Pins('L20', dir = 'i')),
			Subsignal('rx_ctl',  Pins('U19', dir = 'i')),
			Subsignal('rx_data', Pins('T20 U20 T19 R18', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS25')
		),
		Resource('eth_sgmii', 0,
			Subsignal('rst',  PinsN('U17', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS25')),
			Subsignal('mdc',  Pins('T18', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS25')),
			Subsignal('mdio', Pins('U18', dir = 'io'), Attrs(IO_TYPE = 'LVCMOS25')),
			Subsignal('tx',   DiffPairs('W13', 'W14', dir = 'o')),
			Subsignal('rx',   DiffPairs('Y14', 'Y15', dir = 'i')),
		),

		Resource(
			'eth_clk125', 1, Pins('J20', dir = 'i'), Clock(125e6), Attrs(IO_TYPE = 'LVCMOS25')
		),
		Resource(
			'eth_clk125_pll', 1, Pins('C18', dir = 'i'), Clock(125e6), Attrs(IO_TYPE = 'LVCMOS25')
		), # NC by default
		Resource('eth_rgmii', 1,
			Subsignal('rst',     PinsN('F20', dir = 'o')),
			Subsignal('mdc',     Pins('G19', dir = 'o')),
			Subsignal('mdio',    Pins('H20', dir = 'io')),
			Subsignal('tx_clk',  Pins('C20', dir = 'o')),
			Subsignal('tx_ctrl', Pins('E19', dir = 'o')),
			Subsignal('tx_data', Pins('J17 J16 D19 D20', dir = 'o')),
			Subsignal('rx_clk',  Pins('J19', dir = 'i')),
			Subsignal('rx_ctrl', Pins('F19', dir = 'i')),
			Subsignal('rx_data', Pins('G18 G16 H18 H17', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS25')
		),
		Resource('eth_sgmii', 1,
			Subsignal('rst',  PinsN('F20', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS25')),
			Subsignal('mdc',  Pins('G19', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS25')),
			Subsignal('mdio', Pins('H20', dir = 'io'), Attrs(IO_TYPE = 'LVCMOS25')),
			Subsignal('tx',   DiffPairs('W17', 'W18', dir = 'o')),
			Subsignal('rx',   DiffPairs('Y16', 'Y17', dir = 'i')),
		),

		Resource('ddr3', 0,
			Subsignal('rst',    PinsN('N4', dir = 'o')),
			Subsignal('clk',    DiffPairs('M4', 'N5', dir = 'o'), Attrs(IO_TYPE = 'SSTL135D_I')),
			Subsignal('clk_en', Pins('N2', dir = 'o')),
			Subsignal('cs',     PinsN('K1', dir = 'o')),
			Subsignal('we',     PinsN('M1', dir = 'o')),
			Subsignal('ras',    PinsN('P1', dir = 'o')),
			Subsignal('cas',    PinsN('L1', dir = 'o')),
			Subsignal('a',      Pins('P2 C4 E5 F5 B3 F4 B5 E4 C5 E3 D5 B4 C3', dir = 'o')),
			Subsignal('ba',     Pins('P5 N3 M3', dir = 'o')),
			Subsignal(
				'dqs',    DiffPairs('K2 H4', 'J1 G5', dir = 'io'),
				Attrs(IO_TYPE = 'SSTL135D_I', DIFFRESISTOR = '100', TERMINATION = 'OFF')
			),
			Subsignal('dq',     Pins('L5 F1 K4 G1 L4 H1 G2 J3 D1 C1 E2 C2 F3 A2 E1 B1', dir = 'io')),
			Attrs(TERMINATION = '75'),
			Subsignal('dm',     Pins('J4 H5', dir = 'o')),
			Subsignal('odt',    Pins('L2', dir = 'o')),
			Attrs(IO_TYPE = 'SSTL135_I', SLEWRATE = 'FAST')
		)
	]
	connectors = [
		Connector('expcon', 1, '''
		-   -   -   B19 B12 B9  E6  D6  E7  D7  B11 B6  E9  D9  B8  C8  D8  E8  C7  C6
		-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -
		'''), # X3
		Connector('expcon', 2, '''
		A8  -   A12 A13 B13 C13 D13 E13 A14 C14 D14 E14 D11 C10 A9  B10 D12 E12 -   -
		B15 -   C15 -   D15 -   E15 A16 B16 -   C16 D16 B17 -   C17 A17 B18 A7  A18 -
		'''), # X4
	]

	@property
	def file_templates(self) -> dict[str, str]:
		return {
			**super().file_templates,
			'{{name}}-openocd.cfg': r'''
			interface ftdi
			{# FTDI descriptors is identical between non-5G and 5G recent Versa boards #}
			ftdi_device_desc 'Lattice ECP5_5G VERSA Board'
			ftdi_vid_pid 0x0403 0x6010
			ftdi_channel 0
			ftdi_layout_init 0xfff8 0xfffb
			reset_config none
			adapter_khz 25000

			# ispCLOCK device (unusable with openocd and must be bypassed)
			#jtag newtap ispclock tap -irlen 8 -expected-id 0x00191043
			# ECP5 device
			{% if '5G' in platform.device -%}
			jtag newtap ecp5 tap -irlen 8 -expected-id 0x81112043 ; # LFE5UM5G-45F
			{% else -%}
			jtag newtap ecp5 tap -irlen 8 -expected-id 0x01112043 ; # LFE5UM-45F
			{% endif %}
			'''
		}

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		openocd = environ.get('OPENOCD', 'openocd')
		with products.extract(f'{name}-openocd.cfg', f'{name}.svf') as files:
			(config_filename, vector_filename) = files
			check_call([openocd,
				'-f', config_filename,
				'-c', f'transport select jtag; init; svf -quiet {vector_filename}; exit'
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	VersaECP5Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                               import Attrs, Clock, Connector, Pins, Resource
from torii.build.run                           import BuildProducts
from torii.platform.resources                  import (
	ButtonResources, LEDResources, SPIFlashResources, SwitchResources, UARTResource
)
from torii.platform.vendor.lattice.machxo_2_3l import MachXO3LPlatform

__all__ = (
	'MachXO3SKPlatform',
)

class MachXO3SKPlatform(MachXO3LPlatform):
	device      = 'LCMXO3LF-6900C'
	package     = 'BG256'
	speed       = '5'
	default_clk = 'clk12'

	pretty_name = 'MachXO3SK'
	description = 'Lattice MachXO3LF Starter Kit'

	resources   = [
		Resource(
			'clk12', 0, Pins('C8', dir = 'i'),
			Clock(12e6), Attrs(IO_TYPE = 'LVCMOS33')
		),

		UARTResource(0,
			rx = 'A11', tx = 'C11', rts = 'F10', cts = 'D11', dtr = 'B11', dsr = 'A12', dcd = 'B13', ri = 'A14',
			attrs = Attrs(IO_TYPE = 'LVCMOS33'), role = 'dce'
		), # need to solder R14-R18, R20-R22

		*LEDResources(
			pins = 'H11 J13 J11 L12 K11 L13 N15 P16',
			invert = True, attrs = Attrs(IO_TYPE = 'LVCMOS33')
		), # D9..D2

		*ButtonResources(pins = 'B3', invert = True, attrs = Attrs(IO_TYPE = 'LVCMOS33')),

		*SwitchResources(pins = 'N2 P1 M3 N1',
			invert = True, attrs = Attrs(IO_TYPE = 'LVCMOS33')
		), # SW2

		*SPIFlashResources(0,
			cs_n = 'R5', clk = 'P6', copi = 'T13', cipo = 'T6',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
	]
	connectors  = [
		Connector('j', 3, # J3
			'-  -  A13 C13 F8  B12 C12 E11 E10 D10 '
			'-  -  F9  C10 E8  E9  E7  D8  D7  C7  '
			'-  -  C5  D6  E6  C4  A10 F7  D9  B9  '
			'-  -  B6  B7  B5  A5  B4  A4  -   A3  '
		),
		Connector('j', 4, # J4
			'-  -  K12 K13 M14 N14 L14 N16 M15 M16 '
			'-  -  L15 L16 K14 K16 K15 J14 H14 J15 '
			'-  -  J16 H15 H16 G15 G16 F15 F16 E15 '
			'-  -  E16 E14 D16 C15 D14 F14 G14 B16 '
		),
		Connector('j', 6, # J6
			'-  -  T12 T14 R11 R13 T11 M11 P11 N10 '
			'-  -  T10 P10 R9  R10 T9  N9  P9  M8  '
			'-  -  T8  L8  P8  M6  R7  R8  P7  T7  '
			'-  -  L7  R6  N6  T5  R4  P4  T3  T4  '
		),
		Connector('j', 8, # J8
			'-  -  H6  N3  M2  M1  L2  L1  L3  L5  '
			'-  -  K4  J1  K1  J2  J3  H3  H2  H1  '
			'-  -  G2  G1  F2  F1  E2  E1  D2  D1  '
			'-  -  C2  C1  G3  B1  D3  E3  F3  F5  '
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		openFPGALoader = environ.get('OPENFPGALOADER', 'openFPGALoader')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([openFPGALoader, bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	MachXO3SKPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import Attrs, Clock, Connector, Pins, PinsN, Resource, Subsignal
from torii.build.run              import BuildProducts
from torii.hdl.ir                 import Fragment
from torii.platform.resources     import (
	ButtonResources, Display7SegResource, I2CResource, LEDResources, PS2Resource, SDRAMResource, UARTResource, VGAResource
)
from torii.platform.vendor.altera import AlteraPlatform

__all__ = (
	'RZEasyFPGAA2_2Platform',
)

class RZEasyFPGAA2_2Platform(AlteraPlatform):
	device      = 'EP4CE6' # Cyclone IV 6K LEs
	package     = 'E22'    # EQFP 144 pins
	speed       = 'C8'
	default_clk = 'clk50'  # 50MHz builtin clock
	default_rst = 'rst'

	pretty_name = 'RZ-EasyFPGA A2.2'
	description = 'RZ-EasyFPGA Altera Cyclone IV Development Board'

	resources   = [
		# Clock
		Resource(
			'clk50', 0, Pins('23', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),

		# Reset switch, located on the lower left of the board.
		Resource('rst', 0, PinsN('25', dir = 'i'), Attrs(io_standard = '3.3-V LVTTL')),

		# LEDs, located on the bottom of the board.
		*LEDResources(
			pins = '87 86 85 84', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		# Buttons, located on the bottom of the board, right of the LEDs.
		*ButtonResources(
			pins = '88 89 90 91', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		# Connections to the SKHynix RAM chip on board.
		SDRAMResource(0,
			clk = '43', cs_n = '72', we_n = '69', ras_n = '71', cas_n = '70',
			ba = '73 74', a = '76 77 80 83 68 67 66 65 64 60 75 59',
			dq = '28 30 31 32 33 34 38 39 54 53 52 51 50 49 46 44',
			dqm = '42 55', attrs = Attrs(io_standard = '3.3-V LVCMOS')
		),

		# VGA connector, located on the right of the board.
		VGAResource(0,
			r = '106', g = '105', b = '104',
			hs = '101', vs = '103'
		),

		# 4 digit 7 segment display, located on top of the board.
		Display7SegResource(0,
			a = '128', b = '121', c = '125', d = '129', e = '132', f = '126', g = '124', dp = '127',
			invert = True
		),

		Resource('display_7seg_ctrl', 0,
			Subsignal('en', Pins('133 135 136 137', dir = 'o', invert = True)),
		),

		# PS2 port, located on upper right of the board.
		PS2Resource(0, clk = '119', dat = '120'),

		# LM75 temperature sensor
		I2CResource(0, scl = '112', sda = '113'),

		# AT24C08 EEPROM
		I2CResource(1, scl = '99' , sda = '98' ),

		# Buzzer
		Resource('buzzer', 0, PinsN('110', dir = 'o')),

		# Serial port, located above the VGA port.
		UARTResource(0, tx = '114', rx = '115'),

		# LCD connector, located above the 7 segment display.
		Resource('lcd_hd44780', 0,
			Subsignal('rs', Pins('141', dir = 'o')),
			Subsignal('rw', Pins('138', dir = 'o')),
			Subsignal('e' , Pins('143', dir = 'o')),
			Subsignal('d' , Pins('142 1 144 3 2 10 7 11', dir = 'io')),
		),

		# IR receiver, located right of the buttons.
		Resource('cir', 0,
			Subsignal('rx', Pins('100', dir = 'i'))
		),
	]

	connectors  = [
		# Located above the chip.
		Connector('gpio', 0,
			'-   -   11  7   2   144 142 138 136 133 129 127 125 121 119 114 112 110 -  '
			'-   -   24  10  3   1   143 141 137 135 132 128 126 124 120 115 113 111 -  '),

		# Located right of the chip.
		Connector('gpio', 1,
			'-   -  '
			'106 105'
			'104 103'
			'101 100'
			'99  98 '
			'91  90 '
			'89  88 '
			'87  86 '
			'85  84 '
			'-   -  '),

		# Located below the chip.
		Connector('gpio', 2,
			'30 32 34 39 43 46 50 52 54 58 60 65 67 71 73 75 77 83 -  -  - '
			'28 31 33 38 42 44 51 53 55 59 64 66 68 70 72 74 76 80 -  -  - '
		),
	]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs):
		overrides = {
			'add_settings':
				r'''
				set_global_assignment -name CYCLONEII_RESERVE_NCEO_AFTER_CONFIGURATION 'USE AS REGULAR IO'
				'''
		}
		return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	RZEasyFPGAA2_2Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

import subprocess

from torii.build                  import Attrs, Clock, Connector, Pins, Resource
from torii.platform.resources     import (
	ButtonResources, LEDResources, RGBLEDResource, SPIFlashResources, SRAMResource, UARTResource
)
from torii.platform.vendor.xilinx import XilinxPlatform

'''
Example Usage:
	platform = CModA7_35Platform(toolchain = 'Symbiflow')
	platform.build(Top(), do_program = True)

Supported programmer:
	openFPGALoader
'''

__all__ = (
	'CmodA7_15Platform',
	'CmodA7_35Platform',
)

class _CmodA7Platform(XilinxPlatform):
	package     = 'cpg236'
	speed       = '1'
	default_clk = 'clk12'
	resources   = [
		Resource(
			'clk12', 0, Pins('L17', dir = 'i'), Clock(12e6), Attrs(IOSTANDARD = 'LVCMOS33')
		),

		*LEDResources(pins = 'A17 C16', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),

		RGBLEDResource(0,
			r = 'C17', g = 'B16', b = 'B17', invert = True,
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		*ButtonResources(pins = 'A18 B18', attrs = Attrs(IOSTANDARD='LVCMOS33')),

		UARTResource(0, rx = 'J17', tx = 'J18', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),

		*SPIFlashResources(0,
			cs_n = 'K19', clk = 'E19', copi = 'D19', cipo = 'D18', wp_n = 'G18',
			hold_n = 'F18',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		SRAMResource(0,
			cs_n = 'N19', oe_n = 'P19', we_n = 'R19',
			a = 'M18 M19 K17 N17 P17 P18 R18 W19 U19 V19 W18 T17 T18 U17 U18 V16 W16 W17 V15',
			d = 'W15 W13 W14 U15 U16 V13 V14 U14'
		),

		# One-wire interface to crypto authentication device
		# May not be populated on the board
		Resource('atsha204a', 0, Pins('D17', dir = 'io'), Attrs(IOSTANDARD = 'LVCMOS33'))
	]
	connectors  = [
		Connector('pmod', 0, 'G17 G19 N18 L18 - - H17 H19 J19 K18 - -'), # JA

		# Pin 24/25 are VCC and GND
		# Pin 15/16 are analog (XADC)
		Connector('gpio', 0,
			'M3  L3 A16  K3 C15  H1 A15 B15 A14  J3  J1  K2 '
			'L1  L2   -   -  M1  N3  P3  M2  N1  N2  P1   - '
			' -  R3  T3  R2  T1  T2  U1  W2  V2  W3  V3  W5 '
			'V4  U4  V5  W4  U5  U2  W6  U3  U7  W7  U8  V8 '
		),

		Connector('xadc', 0, {
			'vaux4_n':  'G2',
			'vaux4_p':  'G3',
			'vaux12_n': 'J2',
			'vaux12_p': 'H',
		})
	]

	def toolchain_program(self, products, name):
		with products.extract(f'{name}.bit') as bitstream_filename:
			subprocess.check_call([
				'openFPGALoader', '-c', 'digilent', '--fpga-part', self.device, bitstream_filename
			])


class CmodA7_15Platform(_CmodA7Platform):
	device      = 'xc7a15t'

	pretty_name = 'Cmod A7-15T'
	description = 'Digilent Cmod A7-15T Xilinx Artix7-15T FPGA Module'

class CmodA7_35Platform(_CmodA7Platform):
	device      = 'xc7a35t'

	pretty_name = 'Cmod A7-35T'
	description = 'Digilent Cmod A7-15T Xilinx Artix7-35T FPGA Module'

if __name__ == '__main__':
	from ..test.blinky import Blinky
	CmodA7_35Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Attrs, Clock, Connector, Pins, Resource
from torii.build.run                     import BuildProducts
from torii.platform.resources            import LEDResources, SPIFlashResources
from torii.platform.vendor.lattice.ice40 import ICE40Platform

__all__ = (
	'ICE40HX1KBlinkEVNPlatform',
)

class ICE40HX1KBlinkEVNPlatform(ICE40Platform):
	device      = 'iCE40HX1K'
	package     = 'VQ100'
	default_clk = 'clk3p3'

	pretty_name = 'ICE40HX1KEVN'
	description = 'Lattice iCE40-HX1K Evaluation Board'

	resources   = [
		Resource(
			'clk3p3', 0, Pins('13', dir = 'i'), Clock(3.3e6),
			Attrs(GLOBAL = True, IO_STANDARD = 'SB_LVCMOS')
		),

		*LEDResources(pins = '59 56 53 51', attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),

		Resource('touch', 0, Pins('60'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('touch', 1, Pins('57'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('touch', 2, Pins('54'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('touch', 3, Pins('52'), Attrs(IO_STANDARD = 'SB_LVCMOS')),

		*SPIFlashResources(0,
			cs_n = '49', clk = '48', copi = '45', cipo = '46',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
	]
	connectors  = [
		Connector('pmod',  1, '10  9  8  7 - -  4  3  2  1 - -'), # J1
		Connector('pmod',  5, '40 42 62 64 - - 37 41 63 45 - -'), # J5
		Connector('pmod',  6, '25 24 21 20 - - 26 27 28 33 - -'), # J6
		Connector('pmod', 11, '49 45 46 48 - -'), # J11
		Connector('pmod', 12, '59 56 53 51 - -'), # J12
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		iceburn = environ.get('ICEBURN', 'iCEburn')
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([iceburn, '-evw', bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	ICE40HX1KBlinkEVNPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildPlan, BuildProducts
from torii.hdl.ir                       import Fragment
from torii.platform.resources           import DirectUSBResource, RGBLEDResource, SDCardResources, SPIFlashResources
from torii.platform.vendor.lattice.ecp5 import ECP5Platform

__all__ = (
	'OrangeCrabR0_1Platform',
)

class OrangeCrabR0_1Platform(ECP5Platform):
	device      = 'LFE5U-25F'
	package     = 'MG285'
	speed       = '8'
	default_clk = 'clk'

	pretty_name = 'OrangeCrab R0.1'
	description = 'OrangeCrab R0.1 Lattice ECP5-25F Development Board'

	resources   = [
		Resource('clk', 0, Pins('A9', dir = 'i'), Clock(48e6), Attrs(IO_TYPE = 'LVCMOS33')),

		# Used to reload FPGA configuration.
		# Can enter USB bootloader by assigning button 0 to program.
		Resource('program', 0, PinsN('R16', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS33')),

		RGBLEDResource(0,
			r = 'V17', g = 'T17', b = 'J3', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),

		*SPIFlashResources(0,
			cs_n = 'U17', clk = 'U16', cipo = 'T18', copi = 'U18', wp_n = 'R18', hold_n = 'N18',
			attrs = Attrs(IO_TYPE = 'LVCMOS33'),
		),

		Resource('ddr3', 0,
			Subsignal('rst',     PinsN('B1', dir = 'o')),
			Subsignal('clk',     DiffPairs('J18', 'K18', dir = 'o'), Attrs(IO_TYPE = 'SSTL135D_I')),
			Subsignal('clk_en',  Pins('D6', dir = 'o')),
			Subsignal('cs',      PinsN('A12', dir = 'o')),
			Subsignal('we',      PinsN('B12', dir = 'o')),
			Subsignal('ras',     PinsN('C12', dir = 'o')),
			Subsignal('cas',     PinsN('D13', dir = 'o')),
			Subsignal('a',       Pins('A4 D2 C3 C7 D3 D4 D1 B2 C1 A2 A7 C2 C4', dir = 'o')),
			Subsignal('ba',      Pins('B6 B7 A6', dir = 'o')),
			Subsignal('dqs',     DiffPairs('G18 H17', 'B15 A16', dir = 'io'),
				Attrs(IO_TYPE = 'SSTL135D_I', TERMINATION = 'OFF', DIFFRESISTOR = '100')
			),
			Subsignal('dq',      Pins('F17 F16 G15 F15 J16 C18 H16 F18 C17 D15 B17 C16 A15 B13 A17 A13', dir = 'io'),
				Attrs(TERMINATION = '75')
			),
			Subsignal('dm',      Pins('G16 D16', dir = 'o')),
			Subsignal('odt',     Pins('C13', dir = 'o')),
			Attrs(IO_TYPE = 'SSTL135_I', SLEWRATE = 'FAST')
		),

		Resource('ddr3_pseudo_power', 0,
			# pseudo power pins, leave these at their default value
			Subsignal('vcc_virtual', PinsN('A3 B18 C6 C15 D17 D18 K15 K16 K17', dir = 'o')),
			Subsignal('gnd_virtual', Pins('L15 L16 L18', dir = 'o')),
			Attrs(IO_TYPE = 'SSTL135_II', SLEWRATE = 'FAST')
		),

		*SDCardResources(0,
			dat0 = 'J1', dat1 = 'K3', dat2 = 'L3', dat3 = 'M1', clk = 'K1', cmd = 'K2', cd = 'L1',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', SLEWRATE = 'FAST')
		),

		DirectUSBResource(0, d_p = 'N1', d_n = 'M2', pullup = 'N2', attrs = Attrs(IO_TYPE = 'LVCMOS33'))
	]
	connectors = [
		Connector('io', 0, {
			'0':    'N17',
			'1':    'M18',
			'5':    'B10',
			'6':    'B9',
			'9':    'C8',
			'10':   'B8',
			'11':   'A8',
			'12':   'H2',
			'13':   'J2',
			'cipo': 'N15',
			'copi': 'N16',
			'sck':  'R17',
			'scl':  'C9',
			'sda':  'C10'
		}),
		Connector('mcu', 0, {
			'0':    'A10',
			'1':    'C11',
			'2':    'A11',
			'3':    'B11',
		}),
	]

	@property
	def required_tools(self) -> list[str]:
		return super().required_tools + [
			'dfu-suffix'
		]

	@property
	def command_templates(self) -> list[str]:
		return super().command_templates + [
			r'''
			{{invoke_tool('dfu-suffix')}}
				-v 1209 -p 5af0 -a {{name}}.bit
			'''
		]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = dict(ecppack_opts = '--compress --freq 38.8')
		overrides.update(kwargs)
		return super().toolchain_prepare(fragment, name, **overrides)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		dfu_util = environ.get('DFU_UTIL', 'dfu-util')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([dfu_util, '-D', bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	OrangeCrabR0_1Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import Attrs, Clock, Connector, Pins, Resource
from torii.build.run              import BuildProducts
from torii.platform.resources     import (
	ButtonResources, Display7SegResource, LEDResources, PS2Resource, SDCardResources, SDRAMResource,
	SwitchResources, VGAResource
)
from torii.platform.vendor.altera import AlteraPlatform

__all__ = (
	'DE0CVPlatform',
)

class DE0CVPlatform(AlteraPlatform):
	device      = '5CEBA4' # Cyclone V 49K LEs
	package     = 'F23'    # FBGA-484
	speed       = 'C7'
	default_clk = 'clk50'

	pretty_name = 'DE0-CV'
	description = 'terasIC DE0-CV Altera Cyclone V Development Board'

	resources   = [
		Resource(
			'clk50', 0, Pins('M9', dir = 'i'),  Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 1, Pins('H13', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 2, Pins('E10', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 3, Pins('V15', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),

		*LEDResources(
			pins = 'AA2 AA1 W2 Y3 N2 N1 U2 U1 L2 L1', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*ButtonResources(
			pins = 'U7 W9 M7 M6', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*SwitchResources(
			pins = 'U13 V13 T13 T12 AA15 AB15 AA14 AA13 AB13 AB12',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(0,
			a = 'U21', b = 'V21', c = 'W22', d = 'W21', e = 'Y22', f = 'Y21', g = 'AA22', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(1,
			a = 'AA20', b = 'AB20', c = 'AA19', d = 'AA18', e = 'AB18', f = 'AA17', g = 'U22', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')),
		Display7SegResource(2,
			a = 'Y19', b = 'AB17', c = 'AA10', d = 'Y14', e = 'V14', f = 'AB22', g = 'AB21', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(3,
			a = 'Y16', b = 'W16', c = 'Y17', d = 'V16', e = 'U17', f = 'V18', g = 'V19', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(4,
			a = 'U20', b = 'Y20', c = 'V20', d = 'U16', e = 'U15', f = 'Y15', g = 'P9', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(5,
			a = 'N9', b = 'M8', c = 'T14', d = 'P14', e = 'C1', f = 'C2', g = 'W19', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		VGAResource(0,
			r = 'A9 B10 C9 A5',
			g = 'L7 K7 J7 J8',
			b = 'B6 B7 A8 A7',
			hs = 'H8', vs = 'G8',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		PS2Resource(0, # Keyboard
			clk = 'D3', dat = 'G2', attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		PS2Resource(1, # Mouse
			clk = 'E2', dat = 'G1', attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		*SDCardResources(0,
			clk = 'H11', cmd = 'B11', dat0 = 'K9', dat1 = 'D12', dat2 = 'E12', dat3 = 'C11',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		SDRAMResource(0,
			clk = 'AB11', cke = 'R6', cs_n = 'U6', we_n = 'AB5', ras_n = 'AB6', cas_n = 'V6',
			ba = 'T7 AB7', a = 'W8 T8 U11 Y10 N6 AB10 P12 P7 P8 R5 U8 P6 R7',
			dq = 'Y9 T10 R9 Y11 R10 R11 R12 AA12 AA9 AB8 AA8 AA7 V10 V9 U10 T9', dqm = 'U12 N8',
			attrs = Attrs(io_standard = '3.3-V LVCMOS')
		),
	]
	connectors  = [
		Connector('j', 1,
			'N16  B16  M16  C16  D17  K20  K21  K22  M20  M21  '
			'-    -    N21  R22  R21  T22  N20  N19  M22  P19  '
			'L22  P17  P16  M18  L18  L17  L19  K17  -    -    '
			'K19  P18  R15  R17  R16  T20  T19  T18  T17  T15  '
		),
		Connector('j', 2,
			'H16  A12  H15  B12  A13  B13  C13  D13  G18  G17  '
			'-    -    H18  J18  J19  G11  H10  J11  H14  A15  '
			'J13  L8   A14  B15  C15  E14  E15  E16  -    -    '
			'F14  F15  F13  F12  G16  G15  G13  G12  J17  K16  '
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	DE0CVPlatform().build(Blinky(), do_program = True)



#!/usr/bin/env python3
# SPDX-License-Identifier: BSD-2-Clause

from pathlib    import Path

from setuptools import find_packages, setup

REPO_ROOT   = Path(__file__).parent
README_FILE = (REPO_ROOT / 'README.md')

def scm_version():
	def local_scheme(version):
		if version.tag and not version.distance:
			return version.format_with('')
		else:
			return version.format_choice('+{node}', '+{node}.dirty')
	return {
		'relative_to'   : __file__,
		'version_scheme': 'guess-next-dev',
		'local_scheme'  : local_scheme
	}

setup(
	name             = 'torii-boards',
	use_scm_version  = scm_version(),
	author           = ', '.join([
		'Aki Van Ness',
		'Rachel Mant',
	]),
	author_email     = ', '.join([
		'aki@lethalbit.net',
		'git@dragonmux.network',
	]),
	description      = 'Board and connector definitions for Torii-HDL',
	license          = ' BSD-2-Clause',
	python_requires  = '~=3.10',
	zip_safe         = True,
	url              = 'https://torii-boards.shmdn.link/',

	long_description = README_FILE.read_text(),
	long_description_content_type = 'text/markdown',

	setup_requires   = [
		'wheel',
		'setuptools',
		'setuptools_scm'
	],

	install_requires = [
		'torii>=0.8.0,<1.0',
	],

	extras_require   = {
		'dev': [
			'nox'
		],
	},

	packages         = find_packages(
		where   = '.',
		exclude = (
			'tests',
			'tests.*'
		)
	),
	package_data      = {
		'torii_boards': [
			'py.typed'
		],
	},

	classifiers       = [
		'Development Status :: 4 - Beta',

		'Intended Audience :: Developers',
		'Intended Audience :: Information Technology',
		'Intended Audience :: Science/Research',

		'License :: OSI Approved :: BSD License',

		'Operating System :: MacOS :: MacOS X',
		'Operating System :: Microsoft :: Windows',
		'Operating System :: POSIX :: Linux',

		'Programming Language :: Python :: 3.10',
		'Programming Language :: Python :: 3.11',
		'Programming Language :: Python :: 3.12',
		'Programming Language :: Python :: 3.13',

		'Topic :: Scientific/Engineering',
		'Topic :: Scientific/Engineering :: Electronic Design Automation (EDA)',
		'Topic :: Software Development',
		'Topic :: Software Development :: Embedded Systems',
		'Topic :: Software Development :: Libraries',

		'Typing :: Typed',
	],

	project_urls     = {
		'Documentation': 'https://torii-boards.shmdn.link/',
		'Source Code'  : 'https://github.com/shrine-maiden-heavy-industries/torii-boards',
		'Bug Tracker'  : 'https://github.com/shrine-maiden-heavy-industries/torii-boards/issues',
	},
)



# SPDX-License-Identifier: BSD-3-Clause

import shutil
from pathlib        import Path

import nox
from setuptools_scm import ScmVersion, get_version

ROOT_DIR  = Path(__file__).parent

BUILD_DIR = (ROOT_DIR  / 'build')
CNTRB_DIR = (ROOT_DIR  / 'contrib')
DOCS_DIR  = (ROOT_DIR  / 'docs')
DIST_DIR  = (BUILD_DIR / 'dist')

# Default sessions to run
nox.options.sessions = (
	'test',
	'lint',
	'typecheck'
)

def torii_boards_version() -> str:
	def scheme(version: ScmVersion) -> str:
		if version.tag and not version.distance:
			return version.format_with('')
		else:
			return version.format_choice('+{node}', '+{node}.dirty')

	return get_version(
		root           = str(ROOT_DIR),
		version_scheme = 'guess-next-dev',
		local_scheme   = scheme,
		relative_to    = __file__
	)

@nox.session(reuse_venv = True)
def test(session: nox.Session) -> None:
	session.install(
		'git+https://github.com/shrine-maiden-heavy-industries/torii-hdl.git#egg=torii'
	)
	session.install('.')
	session.run(
		'python', '-m', 'unittest', 'discover',
		'-s', '.'
	)

@nox.session(name = 'build-docs')
def build_docs(session: nox.Session) -> None:
	out_dir = (BUILD_DIR / 'docs')
	shutil.rmtree(out_dir, ignore_errors = True)
	session.install('-r', str(DOCS_DIR / 'requirements.txt'))
	session.install(
		'git+https://github.com/shrine-maiden-heavy-industries/torii-hdl.git#egg=torii'
	)
	session.install('.')
	session.run('sphinx-build', '-b', 'html', str(DOCS_DIR), str(out_dir))

@nox.session(name = 'linkcheck-docs')
def linkcheck_docs(session: nox.Session) -> None:
	out_dir = (BUILD_DIR / 'docs-linkcheck')
	shutil.rmtree(out_dir, ignore_errors = True)
	session.install('-r', str(DOCS_DIR / 'requirements.txt'))
	session.install('.')
	session.run('sphinx-build', '-b', 'linkcheck', str(DOCS_DIR), str(out_dir))

@nox.session
def typecheck(session: nox.Session) -> None:
	out_dir = (BUILD_DIR / 'mypy')
	out_dir.mkdir(parents = True, exist_ok = True)

	session.install('mypy')
	session.install('lxml')
	session.install('.')
	session.run(
		'mypy', '--non-interactive', '--install-types', '--pretty',
		'--cache-dir', str((out_dir / '.mypy-cache').resolve()),
		'--config-file', str((CNTRB_DIR / '.mypy.ini').resolve()),
		'-p', 'torii_boards', '--html-report', str(out_dir.resolve())
	)

@nox.session
def lint(session: nox.Session) -> None:
	session.install('flake8')
	session.run(
		'flake8', '--config', str((CNTRB_DIR / '.flake8').resolve()), './torii_boards'
	)

@nox.session
def dist(session: nox.Session) -> None:
	session.install('build')
	session.run(
		'python', '-m', 'build',
		'-o', str(DIST_DIR)
	)

@nox.session
def upload(session: nox.Session) -> None:
	session.install('twine')
	dist(session)
	session.log(f'Uploading torii-boards-{torii_boards_version()} to PyPi')
	session.run(
		'python', '-m', 'twine',
		'upload', f'{DIST_DIR}/torii*-{torii_boards_version()}*'
	)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Attrs, Clock, Connector, Pins, Resource
from torii.build.run                     import BuildProducts
from torii.platform.resources            import LEDResources, SPIFlashResources, UARTResource
from torii.platform.vendor.lattice.ice40 import ICE40Platform

__all__ = (
	'ICE40HX8KBEVNPlatform',
)

class ICE40HX8KBEVNPlatform(ICE40Platform):
	device      = 'iCE40HX8K'
	package     = 'CT256'
	default_clk = 'clk12'

	pretty_name = 'ICE40HX8KEVN'
	description = 'Lattice iCE40-HX8K Evaluation Board'

	resources   = [
		Resource(
			'clk12', 0, Pins('J3', dir = 'i'), Clock(12e6),
			Attrs(GLOBAL = True, IO_STANDARD = 'SB_LVCMOS')
		),

		*LEDResources(
			pins = 'C3 B3 C4 C5 A1 A2 B4 B5',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		), # D2..D9

		UARTResource(0,
			rx = 'B10', tx = 'B12', rts = 'B13', cts = 'A15', dtr = 'A16', dsr = 'B14', dcd = 'B15',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS', PULLUP = 1),
			role = 'dce'
		),

		*SPIFlashResources(0,
			cs_n = 'R12', clk = 'R11', copi = 'P12', cipo = 'P11',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
	]
	connectors  = [
		Connector('j', 1, # J1
			'A16 -   A15 B15 B13 B14 -   -   B12 B11 '
			'A11 B10 A10 C9  -   -   A9  B9  B8  A7  '
			'B7  C7  -   -   A6  C6  B6  C5  A5  C4  '
			'-   -   B5  C3  B4  B3  A2  A1  -   -   '),
		Connector('j', 2, # J2
			'-   -   -   R15 P16 P15 -   -   N16 M15 '
			'M16 L16 K15 K16 -   -   K14 J14 G14 F14 '
			'J15 H14 -   -   H16 G15 G16 F15 F16 E14 '
			'-   -   E16 D15 D16 D14 C16 B16 -   -   '),
		Connector('j', 3, # J3
			'R16 -   T15 T16 T13 T14 -   -   N12 P13 '
			'N10 M11 T11 P10 -   -   T10 R10 P8  P9  '
			'T9  R9  -   -   T7  T8  T6  R6  T5  R5  '
			'-   -   R3  R4  R2  T3  T1  T2  -   -   '),
		Connector('j', 4, # J4
			'-   -   -   R1  P1  P2  -   -   N3  N2  '
			'M2  M1  L3  L1  -   -   K3  K1  J2  J1  '
			'H2  J3  -   -   G2  H1  F2  G1  E2  F1  '
			'-   -   D1  D2  C1  C2  B1  B2  -   -   '),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		iceprog = environ.get('ICEPROG', 'iceprog')
		with products.extract(f'{name}.bin') as bitstream_filename:
			# TODO: this should be factored out and made customizable
			check_call([iceprog, '-S', bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	ICE40HX8KBEVNPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run              import BuildPlan, BuildProducts
from torii.hdl.ir                 import Fragment
from torii.platform.resources     import (
	ButtonResources, LEDResources, RGBLEDResource, SPIFlashResources, SPIResource, SwitchResources, UARTResource
)
from torii.platform.vendor.xilinx import XilinxPlatform

__all__ = (
	'ArtyA7_35Platform',
	'ArtyA7_100Platform',
)

class _ArtyA7Platform(XilinxPlatform):
	package     = 'csg324'
	speed       = '1L'
	default_clk = 'clk100'
	default_rst = 'rst'

	resources   = [
		Resource(
			'clk100', 0, Pins('E3', dir = 'i'), Clock(100e6), Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('rst', 0, PinsN('C2', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),

		*LEDResources(pins = 'H5 J5 T9 T10', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),

		RGBLEDResource(0, r = 'G6', g = 'F6', b = 'E1', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		RGBLEDResource(1, r = 'G3', g = 'J4', b = 'G4', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		RGBLEDResource(2, r = 'J3', g = 'J2', b = 'H4', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		RGBLEDResource(3, r = 'K1', g = 'H6', b = 'K2', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),

		*ButtonResources(pins = 'D9  C9  B9  B8 ', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		*SwitchResources(pins = 'A8  C11 C10 A10', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),

		UARTResource(0,
			rx = 'A9', tx = 'D10',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		SPIResource(0,
			cs_n = 'C1', clk = 'F1', copi = 'H1', cipo = 'G1',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource('i2c', 0,
			Subsignal('scl',        Pins('L18', dir = 'io')),
			Subsignal('sda',        Pins('M18', dir = 'io')),
			Subsignal('scl_pullup', Pins('A14', dir = 'o')),
			Subsignal('sda_pullup', Pins('A13', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		*SPIFlashResources(0,
			cs_n = 'L13', clk = 'L16', copi = 'K17', cipo = 'K18', wp_n = 'L14', hold_n = 'M14',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource('ddr3', 0,
			Subsignal('rst',    PinsN('K6', dir = 'o')),
			Subsignal('clk',    DiffPairs('U9', 'V9', dir = 'o'), Attrs(IOSTANDARD = 'DIFF_SSTL135')),
			Subsignal('clk_en', Pins('N5', dir = 'o')),
			Subsignal('cs',     PinsN('U8', dir = 'o')),
			Subsignal('we',     PinsN('P5', dir = 'o')),
			Subsignal('ras',    PinsN('P3', dir = 'o')),
			Subsignal('cas',    PinsN('M4', dir = 'o')),
			Subsignal('a',      Pins('R2 M6 N4 T1 N6 R7 V6 U7 R8 V7 R6 U6 T6 T8', dir = 'o')),
			Subsignal('ba',     Pins('R1 P4 P2', dir = 'o')),
			Subsignal('dqs',    DiffPairs('N2 U2', 'N1 V2', dir = 'io'),
								Attrs(IOSTANDARD = 'DIFF_SSTL135')
			),
			Subsignal('dq',     Pins('K5 L3 K3 L6 M3 M1 L4 M2 V4 T5 U4 V5 V1 T3 U3 R3', dir = 'io'),
								Attrs(IN_TERM = 'UNTUNED_SPLIT_40')
			),
			Subsignal('dm',     Pins('L1 U1', dir = 'o')),
			Subsignal('odt',    Pins('R5', dir = 'o')),
			Attrs(IOSTANDARD = 'SSTL135', SLEW = 'FAST'),
		),

		Resource(
			'eth_clk25', 0, Pins('G18', dir = 'o'), Clock(25e6),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource(
			'eth_clk50', 0, Pins('G18', dir = 'o'), Clock(50e6),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('eth_mii', 0,
			Subsignal('rst',     PinsN('C16', dir = 'o')),
			Subsignal('mdio',    Pins('K13', dir = 'io')),
			Subsignal('mdc',     Pins('F16', dir = 'o')),
			Subsignal('tx_clk',  Pins('H16', dir = 'i')),
			Subsignal('tx_en',   Pins('H15', dir = 'o')),
			Subsignal('tx_data', Pins('H14 J14 J13 H17', dir = 'o')),
			Subsignal('rx_clk',  Pins('F15', dir = 'i')),
			Subsignal('rx_dv',   Pins('G16', dir = 'i'), Attrs(PULLDOWN = 'TRUE')), # strap to select MII
			Subsignal('rx_er',   Pins('C17', dir = 'i')),
			Subsignal('rx_data', Pins('D18 E17 E18 G17', dir = 'i')),
			Subsignal('col',     Pins('D17', dir = 'i')),
			Subsignal('crs',     Pins('G14', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('eth_rmii', 0,
			Subsignal('rst',       PinsN('C16', dir = 'o')),
			Subsignal('mdio',      Pins('K13', dir = 'io')),
			Subsignal('mdc',       Pins('F16', dir = 'o')),
			Subsignal('tx_en',     Pins('H15', dir = 'o')),
			Subsignal('tx_data',   Pins('H14 J14', dir = 'o')),
			Subsignal('rx_crs_dv', Pins('G14', dir = 'i')),
			Subsignal('rx_dv',     Pins('G16', dir = 'i'), Attrs(PULLUP = 'TRUE')), # strap to select RMII
			Subsignal('rx_er',     Pins('C17', dir = 'i')),
			Subsignal('rx_data',   Pins('D18 E17', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		)
	]
	connectors  = [
		Connector('pmod', 0, 'G13 B11 A11 D12 - - D13 B18 A18 K16 - -'), # JA
		Connector('pmod', 1, 'E15 E16 D15 C15 - - J17 J18 K15 J15 - -'), # JB
		Connector('pmod', 2, 'U12 V12 V10 V11 - - U14 V14 T13 U13 - -'), # JC
		Connector('pmod', 3, ' D4  D3  F4  F3 - -  E2  D2  H2  G2 - -'), # JD

		Connector('ck_io', 0, {
			# Outer Digital Header
			'io0':  'V15',
			'io1':  'U16',
			'io2':  'P14',
			'io3':  'T11',
			'io4':  'R12',
			'io5':  'T14',
			'io6':  'T15',
			'io7':  'T16',
			'io8':  'N15',
			'io9':  'M16',
			'io10': 'V17',
			'io11': 'U18',
			'io12': 'R17',
			'io13': 'P17',

			# Inner Digital Header
			'io26': 'U11',
			'io27': 'V16',
			'io28': 'M13',
			'io29': 'R10',
			'io30': 'R11',
			'io31': 'R13',
			'io32': 'R15',
			'io33': 'P15',
			'io34': 'R16',
			'io35': 'N16',
			'io36': 'N14',
			'io37': 'U17',
			'io38': 'T18',
			'io39': 'R18',
			'io40': 'P18',
			'io41': 'N17',

			# Outer Analog Header as Digital IO
			'a0': 'F5',
			'a1': 'D8',
			'a2': 'C7',
			'a3': 'E7',
			'a4': 'D7',
			'a5': 'D5',

			# Inner Analog Header as Digital IO
			'io20': 'B7',
			'io21': 'B6',
			'io22': 'E6',
			'io23': 'E5',
			'io24': 'A4',
			'io25': 'A3'
		}),
		Connector('xadc', 0, {
			# Outer Analog Header
			'vaux4_n':   'C5',
			'vaux4_p':   'C6',
			'vaux5_n':   'A5',
			'vaux5_p':   'A6',
			'vaux6_n':   'B4',
			'vaux6_p':   'C4',
			'vaux7_n':   'A1',
			'vaux7_p':   'B1',
			'vaux15_n':  'B2',
			'vaux15_p':  'B3',
			'vaux0_n':  'C14',
			'vaux0_p':  'D14',

			# Inner Analog Header
			'vaux12_n': 'B7',
			'vaux12_p': 'B6',
			'vaux13_n': 'E6',
			'vaux13_p': 'E5',
			'vaux14_n': 'A4',
			'vaux14_p': 'A3',

			# Power Measurements
			'vsnsuv_n':   'B17',
			'vsnsuv_p':   'B16',
			'vsns5v0_n':  'B12',
			'vsns5v0_p':  'C12',
			'isns5v0_n':  'F14',
			'isns5v0_p':  'F13',
			'isns0v95_n': 'A16',
			'isns0v95_p': 'A15',
		})
	]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = {
			'script_before_bitstream':
				'set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]',
			'script_after_bitstream':
				'write_cfgmem -force -format bin -interface spix4 -size 16 '
				'-loadbit \'up 0x0 {name}.bit\' -file {name}.bin'.format(name = name),
			'add_constraints':
				'''
				set_property INTERNAL_VREF 0.675 [get_iobanks 34]
				set_property CFGBVS VCCO [current_design]
				set_property CONFIG_VOLTAGE 3.3 [current_design]
				'''
		}
		return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import run

		xc3sprog = environ.get('XC3SPROG', 'xc3sprog')
		with products.extract(f'{name}.bit') as bitstream_filename:
			run([xc3sprog, '-c', 'nexys4', bitstream_filename], check = True)


class ArtyA7_35Platform(_ArtyA7Platform):
	device = 'xc7a35ti'

	pretty_name = 'Arty A7-35T'
	description = 'Digilent Arty A7-35T Xilinx Artix7-35 Development Board'

class ArtyA7_100Platform(_ArtyA7Platform):
	device = 'xc7a100ti'

	pretty_name = 'Arty A7-100T'
	description = 'Digilent Arty-A7-100T Xilinx Artix7-100 Development Board'

if __name__ == '__main__':
	from ..test.blinky import Blinky
	ArtyA7_35Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from .zturn_lite_z007s import ZTurnLiteZ007SPlatform

__all__ = (
	'ZTurnLiteZ010Platform',
)

class ZTurnLiteZ010Platform(ZTurnLiteZ007SPlatform):
	device = 'xc7z010'

	pretty_name = 'Z-Turn Lite'
	description = 'Myir Z-Turn Lite Xilinx Zynq-7010 SoC Development Board'
